getAllObjects = [
  {
    "description": "Administrative",
    "id": 1,
    "keyName": "ADMINISTRATIVE",
    "name": "Administrative",
    "permissions": [
      {
        "createDate": None,
        "departmentId": 1,
        "description": "Permission to access account billing system type determination endpoint",
        "id": 5088,
        "key": None,
        "keyName": "ACCOUNT_BILLING_SYSTEM",
        "modifyDate": None,
        "name": "Account Billing System"
      },
      {
        "createDate": None,
        "departmentId": 1,
        "description": "Activate Partner Customer Account",
        "id": 1315,
        "key": "A_16",
        "keyName": "ACTIVATE_PARTNER_ACCOUNT",
        "modifyDate": None,
        "name": "Activate Partner Customer Account"
      },
      {
        "createDate": None,
        "departmentId": 1,
        "description": "Permission to create sub brands",
        "id": 1313,
        "key": "A_13",
        "keyName": "ACCOUNT_BRAND_ADD",
        "modifyDate": None,
        "name": "Add Brand Account"
      },
      {
        "createDate": None,
        "departmentId": 1,
        "description": "Permission to create customer account.",
        "id": 1312,
        "key": "A_14",
        "keyName": "ACCOUNT_CUSTOMER_ADD",
        "modifyDate": None,
        "name": "Add Customer Account"
      },
      {
        "createDate": None,
        "departmentId": 1,
        "description": "Permission to interface with the Automated Brand Migration process",
        "id": 5089,
        "key": None,
        "keyName": "AUTOMATED_BRAND_MIGRATION",
        "modifyDate": None,
        "name": "Automated Brand Migration"
      },
      {
        "createDate": None,
        "departmentId": 1,
        "description": "Modify the account and company profile associated with this account.",
        "id": 1268,
        "key": "A_2",
        "keyName": "COMPANY_EDIT",
        "modifyDate": None,
        "name": "Edit Company Profile"
      },
      {
        "createDate": None,
        "departmentId": 1,
        "description": "Permission to manage account notes",
        "id": 5087,
        "key": None,
        "keyName": "MANAGE_ACCOUNT_NOTE",
        "modifyDate": None,
        "name": "Manage Account Notes"
      },
      {
        "createDate": None,
        "departmentId": 1,
        "description": "Manage e-mail delivery service accounts.",
        "id": 1308,
        "key": "NET_4",
        "keyName": "NETWORK_MESSAGE_DELIVERY_MANAGE",
        "modifyDate": None,
        "name": "Manage E-mail Delivery Service"
      },
      {
        "createDate": None,
        "departmentId": 1,
        "description": "Provides users ability to toggle the EU Supported account flag.",
        "id": 3693,
        "key": "EU_1",
        "keyName": "EU_LIMITED_PROCESSING_MANAGE",
        "modifyDate": None,
        "name": "Manage EU Supported Account Flag"
      },
      {
        "createDate": None,
        "departmentId": 1,
        "description": "Create and manage notification subscribers for usage warnings and overages.",
        "id": 1296,
        "key": "NTF_1",
        "keyName": "NTF_SUBSCRIBER_MANAGE",
        "modifyDate": None,
        "name": "Manage Notification Subscribers"
      },
      {
        "createDate": None,
        "departmentId": 1,
        "description": "Manage users and assign permissions.",
        "id": 1266,
        "key": "A_0",
        "keyName": "USER_MANAGE",
        "modifyDate": None,
        "name": "Manage Users"
      },
      {
        "createDate": None,
        "departmentId": 1,
        "description": "Physically Access a Customer's Colo Cage",
        "id": 1319,
        "key": "DA_2",
        "keyName": "DATACENTER_ROOM_ACCESS",
        "modifyDate": None,
        "name": "Physically Access a Customer's Colo Cage"
      },
      {
        "createDate": None,
        "departmentId": 1,
        "description": "Physically Access a Datacenter",
        "id": 1318,
        "key": "DA_1",
        "keyName": "DATACENTER_ACCESS",
        "modifyDate": None,
        "name": "Physically Access a Datacenter"
      },
      {
        "createDate": None,
        "departmentId": 1,
        "description": "Submit one-time payments for this account.",
        "id": 63,
        "key": "A_4",
        "keyName": "ONE_TIME_PAYMENTS",
        "modifyDate": None,
        "name": "Submit One-Time Payments"
      },
      {
        "createDate": None,
        "departmentId": 1,
        "description": "Update the recurring monthly payment information.",
        "id": 62,
        "key": "A_3",
        "keyName": "UPDATE_PAYMENT_DETAILS",
        "modifyDate": None,
        "name": "Update Payment Details"
      },
      {
        "createDate": None,
        "departmentId": 1,
        "description": "View the account summary page including invoices and payments.",
        "id": 64,
        "key": "A_1",
        "keyName": "ACCOUNT_SUMMARY_VIEW",
        "modifyDate": None,
        "name": "View Account Summary"
      },
      {
        "createDate": None,
        "departmentId": 1,
        "description": "View the account-wide event log history.",
        "id": 1314,
        "key": "A_15",
        "keyName": "USER_EVENT_LOG_VIEW",
        "modifyDate": None,
        "name": "View Event Log"
      }
    ]
  },
  {
    "description": "Sales",
    "id": 2,
    "keyName": "SALES",
    "name": "Sales",
    "permissions": [
      {
        "createDate": None,
        "departmentId": 2,
        "description": "Add new servers to the account.",
        "id": 1267,
        "key": "XX_1",
        "keyName": "SERVER_ADD",
        "modifyDate": None,
        "name": "Add Server"
      },
      {
        "createDate": None,
        "departmentId": 2,
        "description": "Add and upgrade any cloud computing instances on the account.",
        "id": 1303,
        "key": "A_11",
        "keyName": "INSTANCE_UPGRADE",
        "modifyDate": None,
        "name": "Add/Upgrade Cloud Instances"
      },
      {
        "createDate": None,
        "departmentId": 2,
        "description": "Add and upgrade any services on the account.",
        "id": 1271,
        "key": "A_7",
        "keyName": "SERVICE_ADD",
        "modifyDate": None,
        "name": "Add/Upgrade Services"
      },
      {
        "createDate": None,
        "departmentId": 2,
        "description": "Add and upgrade any storage services (StorageLayer) on the account.",
        "id": 1265,
        "key": "A_10",
        "keyName": "ADD_SERVICE_STORAGE",
        "modifyDate": None,
        "name": "Add/Upgrade Storage (StorageLayer)"
      },
      {
        "createDate": None,
        "departmentId": 2,
        "description": "Cancel any servers on the account.",
        "id": 1270,
        "key": "A_6",
        "keyName": "SERVER_CANCEL",
        "modifyDate": None,
        "name": "Cancel Server"
      },
      {
        "createDate": None,
        "departmentId": 2,
        "description": "Cancel any services on the account.",
        "id": 1273,
        "key": "A_9",
        "keyName": "SERVICE_CANCEL",
        "modifyDate": None,
        "name": "Cancel Services"
      },
      {
        "createDate": None,
        "departmentId": 2,
        "description": "Upgrade any servers on the account.",
        "id": 1269,
        "key": "A_5",
        "keyName": "SERVER_UPGRADE",
        "modifyDate": None,
        "name": "Upgrade Server"
      },
      {
        "createDate": None,
        "departmentId": 2,
        "description": "Upgrade Services",
        "id": 1272,
        "key": "A_8",
        "keyName": "SERVICE_UPGRADE",
        "modifyDate": None,
        "name": "Upgrade Services"
      },
      {
        "createDate": None,
        "departmentId": 2,
        "description": "Permission to view billing ACH information.",
        "id": 5085,
        "key": None,
        "keyName": "VIEW_ACH_INFO",
        "modifyDate": None,
        "name": "View Billing ACH Information"
      },
      {
        "createDate": None,
        "departmentId": 2,
        "description": "Permission to view an order with reseller pricing.",
        "id": 5086,
        "key": None,
        "keyName": "VIEW_RESELLER_ORDER",
        "modifyDate": None,
        "name": "View reseller order pricing"
      }
    ]
  },
  {
    "description": "Support",
    "id": 3,
    "keyName": "SUPPORT",
    "name": "Support",
    "permissions": [
      {
        "createDate": None,
        "departmentId": 3,
        "description": "Add new support tickets.",
        "id": 3,
        "key": "T_7",
        "keyName": "TICKET_ADD",
        "modifyDate": None,
        "name": "Add Tickets"
      },
      {
        "createDate": None,
        "departmentId": 3,
        "description": "Edit support tickets.",
        "id": 2,
        "key": "T_8",
        "keyName": "TICKET_EDIT",
        "modifyDate": None,
        "name": "Edit Tickets"
      },
      {
        "createDate": None,
        "departmentId": 3,
        "description": "Search through previous tickets.",
        "id": 203,
        "key": "T_2",
        "keyName": "TICKET_SEARCH",
        "modifyDate": None,
        "name": "Search Tickets"
      },
      {
        "createDate": None,
        "departmentId": 3,
        "description": "View all tickets regardless of which user the ticket is assigned to.",
        "id": 1321,
        "key": "T_6",
        "keyName": "TICKET_VIEW_ALL",
        "modifyDate": None,
        "name": "View All Tickets"
      },
      {
        "createDate": None,
        "departmentId": 3,
        "description": "View tickets assigned to the user.",
        "id": 1,
        "key": "T_1",
        "keyName": "TICKET_VIEW",
        "modifyDate": None,
        "name": "View Tickets"
      }
    ]
  },
  {
    "description": "Security",
    "id": 4,
    "keyName": "SECURITY",
    "name": "Security",
    "permissions": [
      {
        "createDate": None,
        "departmentId": 4,
        "description": "Add, remove, and update certificates (includes the private key).",
        "id": 1325,
        "key": "SE_9",
        "keyName": "SECURITY_CERTIFICATE_MANAGE",
        "modifyDate": None,
        "name": "Manage Certificates (SSL)"
      },
      {
        "createDate": None,
        "departmentId": 4,
        "description": "Create, edit and delete SAML authentication records.",
        "id": 3664,
        "key": "SM_1",
        "keyName": "SAML_AUTHENTICATION_MANAGE",
        "modifyDate": None,
        "name": "Manage SAML Authentication"
      },
      {
        "createDate": None,
        "departmentId": 4,
        "description": "Add, remove, and update SSH keys.",
        "id": 1320,
        "key": "SE_10",
        "keyName": "CUSTOMER_SSH_KEY_MANAGEMENT",
        "modifyDate": None,
        "name": "Manage SSH Keys"
      },
      {
        "createDate": None,
        "departmentId": 4,
        "description": "Request compliance reports.",
        "id": 2442,
        "key": "COM_1",
        "keyName": "REQUEST_COMPLIANCE_REPORT",
        "modifyDate": None,
        "name": "Request Compliance Report"
      },
      {
        "createDate": None,
        "departmentId": 4,
        "description": "View certificates (includes the private key).",
        "id": 1324,
        "key": "SE_8",
        "keyName": "SECURITY_CERTIFICATE_VIEW",
        "modifyDate": None,
        "name": "View Certificates (SSL)"
      },
      {
        "createDate": None,
        "departmentId": 4,
        "description": "Request and view Vulnerability Scans.",
        "id": 1295,
        "key": "SE_7",
        "keyName": "VULN_SCAN_MANAGE",
        "modifyDate": None,
        "name": "Vulnerability Scanning"
      }
    ]
  },
  {
    "description": "Devices",
    "id": 5,
    "keyName": "DEVICES",
    "name": "Devices",
    "permissions": [
      {
        "createDate": None,
        "departmentId": 5,
        "description": "Allows a user to access virtual dedicated hosts",
        "id": 3679,
        "key": "ALL_3",
        "keyName": "ACCESS_ALL_DEDICATEDHOSTS",
        "modifyDate": None,
        "name": "Access Virtual DedicatedHosts"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "Add IP Addresses to a server.",
        "id": 1281,
        "key": "H_6",
        "keyName": "IP_ADD",
        "modifyDate": None,
        "name": "Add IP Addresses"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "Allow a user to access all guests on the account.",
        "id": 1841,
        "key": "ALL_2",
        "keyName": "ACCESS_ALL_GUEST",
        "modifyDate": None,
        "name": "All Guest Access"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "Allow a user to access all hardware on the account.",
        "id": 1821,
        "key": "ALL_1",
        "keyName": "ACCESS_ALL_HARDWARE",
        "modifyDate": None,
        "name": "All Hardware Access"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "Edit hostname and domain name for devices on the account.",
        "id": 1304,
        "key": "H_7",
        "keyName": "HOSTNAME_EDIT",
        "modifyDate": None,
        "name": "Edit Hostname/Domain"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "Allows users to edit Hardware Component Hard Drive Dirty Attribute.",
        "id": 3742,
        "key": None,
        "keyName": "HARDWARE_COMPONENT_DRIVE_DIRTY_ATTRIBUTE_EDIT",
        "modifyDate": None,
        "name": "Hardware Component Hard Drive Dirty Attribute Edit"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "View Host IDS logs.",
        "id": 1294,
        "key": "SE_6",
        "keyName": "HOST_ID_MANAGE",
        "modifyDate": None,
        "name": "Host IDS"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "View IPMI details regarding hardware and issue reboot commands through the portal.",
        "id": 1277,
        "key": "H_2",
        "keyName": "REMOTE_MANAGEMENT",
        "modifyDate": None,
        "name": "IPMI Remote Management"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "Allows users to manage configuration template.",
        "id": 3748,
        "key": None,
        "keyName": "MANAGE_CONFIGURATION_TEMPLATE",
        "modifyDate": None,
        "name": "Manage Configuration Template"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "Allows users to manage customer hardware.",
        "id": 3746,
        "key": None,
        "keyName": "MANAGE_CUSTOMER_HARDWARE",
        "modifyDate": None,
        "name": "Manage Customer Hardware"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "View and edit monitoring information for devices.",
        "id": 1278,
        "key": "H_3",
        "keyName": "MONITORING_MANAGE",
        "modifyDate": None,
        "name": "Manage Device Monitoring"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "Manage Customer Post Provisioning Scripts.",
        "id": 541,
        "key": "SO_8",
        "keyName": "CUSTOMER_POST_PROVISION_SCRIPT_MANAGEMENT",
        "modifyDate": None,
        "name": "Manage Provisioning Scripts"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "Manage Public Image Templates.",
        "id": 1323,
        "key": "I_1",
        "keyName": "PUBLIC_IMAGE_MANAGE",
        "modifyDate": None,
        "name": "Manage Public Images"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "Issue OS reloads and Rescue Kernel for devices.",
        "id": 1279,
        "key": "H_4",
        "keyName": "SERVER_RELOAD",
        "modifyDate": None,
        "name": "OS Reloads and Rescue Kernel"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "View storage details and edit storage passwords.",
        "id": 1283,
        "key": "NAS_2",
        "keyName": "NAS_MANAGE",
        "modifyDate": None,
        "name": "Storage Manage"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "View hardware information such as IP addresses, OS type, p",
        "id": 163,
        "key": "H_1",
        "keyName": "HARDWARE_VIEW",
        "modifyDate": None,
        "name": "View Hardware Details"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "Allows users to view location reservation.",
        "id": 3750,
        "key": None,
        "keyName": "VIEW_LOCATION_RESERVATION",
        "modifyDate": None,
        "name": "View Location Reservation"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "View virtual dedicated host information. ",
        "id": 3684,
        "key": "VH_1",
        "keyName": "DEDICATED_HOST_VIEW",
        "modifyDate": None,
        "name": "View Virtual Dedicated Host Details"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "View virtual server information such as IP addresses, OS type, passwords, e",
        "id": 1302,
        "key": "VG_1",
        "keyName": "VIRTUAL_GUEST_VIEW",
        "modifyDate": None,
        "name": "View Virtual Server Details"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "Allows users to view and edit dedicated host.",
        "id": 6278,
        "key": None,
        "keyName": "MANAGE_DEDICATED_HOST",
        "modifyDate": None,
        "name": "View and edit dedicated host"
      },
      {
        "createDate": None,
        "departmentId": 5,
        "description": "Allows users to view and edit virtual guest data.",
        "id": 6280,
        "key": None,
        "keyName": "MANAGE_VIRTUAL_GUEST",
        "modifyDate": None,
        "name": "View and edit virtual guest"
      }
    ]
  },
  {
    "description": "Network",
    "id": 6,
    "keyName": "NETWORK",
    "name": "Network",
    "permissions": [
      {
        "createDate": None,
        "departmentId": 6,
        "description": "When adding compute (Server or Cloud Instance), a",
        "id": 3682,
        "key": "NET_6",
        "keyName": "PUBLIC_NETWORK_COMPUTE",
        "modifyDate": None,
        "name": "Add Compute with Public Network Port"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "Manage content delivery network account.",
        "id": 1298,
        "key": "CDN_1",
        "keyName": "CDN_ACCOUNT_MANAGE",
        "modifyDate": None,
        "name": "Manage CDN Account"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "Manage content delivery network file transfers.",
        "id": 1299,
        "key": "CDN_2",
        "keyName": "CDN_FILE_MANAGE",
        "modifyDate": None,
        "name": "Manage CDN File Transfers"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "Add, edit, and view DNS records managed by SoftLayer.",
        "id": 1275,
        "key": "DNS_1",
        "keyName": "DNS_MANAGE",
        "modifyDate": None,
        "name": "Manage DNS"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "Manage all firewall rules.",
        "id": 1322,
        "key": "FW_1",
        "keyName": "FIREWALL_RULE_MANAGE",
        "modifyDate": None,
        "name": "Manage Firewall Rules"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "View and edit firewall logs and settings.",
        "id": 1290,
        "key": "SE_2",
        "keyName": "FIREWALL_MANAGE",
        "modifyDate": None,
        "name": "Manage Firewalls"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "Manage IPSEC network tunnels.",
        "id": 250,
        "key": "NET_3",
        "keyName": "NETWORK_TUNNEL_MANAGE",
        "modifyDate": None,
        "name": "Manage IPSEC Network Tunnels"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "Manage load balancers.",
        "id": 1289,
        "key": "LBS_1",
        "keyName": "LOADBALANCER_MANAGE",
        "modifyDate": None,
        "name": "Manage Load Balancers"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "Manage network gateway appliances.",
        "id": 1842,
        "key": "GTW_1",
        "keyName": "GATEWAY_MANAGE",
        "modifyDate": None,
        "name": "Manage Network Gateways"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "Manage network IDs",
        "id": 1293,
        "key": "SE_5",
        "keyName": "NETWORK_IDS_MANAGE",
        "modifyDate": None,
        "name": "Manage Network IDs"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "Manage network subnet routes.",
        "id": 1301,
        "key": "NET_1",
        "keyName": "NETWORK_ROUTE_MANAGE",
        "modifyDate": None,
        "name": "Manage Network Subnet Routes"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "Enable and disable private network VLAN spanning.",
        "id": 1297,
        "key": "NET_2",
        "keyName": "NETWORK_VLAN_SPANNING",
        "modifyDate": None,
        "name": "Manage Network VLAN Spanning"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "Manage port status and speeds for connected devices.",
        "id": 1285,
        "key": "PO_1",
        "keyName": "PORT_CONTROL",
        "modifyDate": None,
        "name": "Manage Port Control"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "Permission to connect and disconnect account with the private endpoint service.",
        "id": 5048,
        "key": None,
        "keyName": "MANAGE_PRIVATE_ENDPOINT_SERVICE",
        "modifyDate": None,
        "name": "Manage Private Endpoint Service"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "Permission to Manage the Public Network",
        "id": 3672,
        "key": "MP_1",
        "keyName": "MANAGE_PUBLIC_NETWORK",
        "modifyDate": None,
        "name": "Manage Public Network"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "Manage security groups.",
        "id": 3678,
        "key": "NET_5",
        "keyName": "MANAGE_SECURITY_GROUPS",
        "modifyDate": None,
        "name": "Manage Security Groups"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "Manage VPN access for all users.",
        "id": 1316,
        "key": "VPN_1",
        "keyName": "VPN_MANAGE",
        "modifyDate": None,
        "name": "VPN Administration"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "View bandwidth statistics and graphs for hardware.",
        "id": 1274,
        "key": "B_1",
        "keyName": "BANDWIDTH_MANAGE",
        "modifyDate": None,
        "name": "View Bandwidth Statistics"
      },
      {
        "createDate": None,
        "departmentId": 6,
        "description": "View content delivery network bandwidth statistics.",
        "id": 1300,
        "key": "CDN_3",
        "keyName": "CDN_BANDWIDTH_VIEW",
        "modifyDate": None,
        "name": "View CDN Bandwidth Statistics"
      }
    ]
  },
  {
    "description": "Software",
    "id": 7,
    "keyName": "SOFTWARE",
    "name": "Software",
    "permissions": [
      {
        "createDate": None,
        "departmentId": 7,
        "description": "View and edit antivirus / spyware logs and settings.",
        "id": 1292,
        "key": "SE_4",
        "keyName": "ANTI_MALWARE_MANAGE",
        "modifyDate": None,
        "name": "Manage Antivirus/Spyware"
      },
      {
        "createDate": None,
        "departmentId": 7,
        "description": "Manage firewall software",
        "id": 1291,
        "key": "SE_3",
        "keyName": "SOFTWARE_FIREWALL_MANAGE",
        "modifyDate": None,
        "name": "Manage Firewall Software"
      },
      {
        "createDate": None,
        "departmentId": 7,
        "description": "Permission to initiate and delete an openstack link.",
        "id": 3514,
        "key": None,
        "keyName": "OPENSTACK_LINK",
        "modifyDate": None,
        "name": "Openstack Link"
      },
      {
        "createDate": None,
        "departmentId": 7,
        "description": "View Customer Software Password",
        "id": 240,
        "key": "SO_9",
        "keyName": "VIEW_CUSTOMER_SOFTWARE_PASSWORD",
        "modifyDate": None,
        "name": "View Customer Software Password"
      },
      {
        "createDate": None,
        "departmentId": 7,
        "description": "View login information for Helm.",
        "id": 1263,
        "key": "SO_3",
        "keyName": "VIEW_HELM",
        "modifyDate": None,
        "name": "View Helm"
      },
      {
        "createDate": None,
        "departmentId": 7,
        "description": "View login information for Plesk.",
        "id": 1262,
        "key": "SO_2",
        "keyName": "VIEW_PLESK",
        "modifyDate": None,
        "name": "View Plesk"
      },
      {
        "createDate": None,
        "departmentId": 7,
        "description": "View login information for QuantaStor.",
        "id": 1317,
        "key": "SO_7",
        "keyName": "VIEW_QUANTASTOR",
        "modifyDate": None,
        "name": "View QuantaStor"
      },
      {
        "createDate": None,
        "departmentId": 7,
        "description": "View login information for Urchin.",
        "id": 1264,
        "key": "SO_4",
        "keyName": "VIEW_URCHIN",
        "modifyDate": None,
        "name": "View Urchin"
      },
      {
        "createDate": None,
        "departmentId": 7,
        "description": "Allows users to view and edit disk image data.",
        "id": 6279,
        "key": None,
        "keyName": "MANAGE_DISK_IMAGE",
        "modifyDate": None,
        "name": "View and edit disk image"
      },
      {
        "createDate": None,
        "departmentId": 7,
        "description": "Allows users to view and edit image template.",
        "id": 6277,
        "key": None,
        "keyName": "IMAGE_TEMPLATE_MANAGE",
        "modifyDate": None,
        "name": "View and edit manage image template"
      },
      {
        "createDate": None,
        "departmentId": 7,
        "description": "Allows users to view and edit software component.",
        "id": 6276,
        "key": None,
        "keyName": "SOFTWARE_MANAGE",
        "modifyDate": None,
        "name": "View and edit software component"
      },
      {
        "createDate": None,
        "departmentId": 7,
        "description": "View login information for cPanel.",
        "id": 1261,
        "key": "SO_1",
        "keyName": "VIEW_CPANEL",
        "modifyDate": None,
        "name": "View cPanel"
      },
      {
        "createDate": None,
        "departmentId": 7,
        "description": "View licenses",
        "id": 1280,
        "key": "H_5",
        "keyName": "LICENSE_VIEW",
        "modifyDate": None,
        "name": "View licenses"
      },
      {
        "createDate": None,
        "departmentId": 7,
        "description": "Allows users to view software account license.",
        "id": 6275,
        "key": None,
        "keyName": "SOFTWARE_LICENSE_MANAGE",
        "modifyDate": None,
        "name": "View software account license"
      }
    ]
  }
]



"""Add a new load balancer protocol."""
import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.CLI.loadbal import protocol_add


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--uuid', help="Load Balancer Uuid.")
@click.option('--frontend', '-f', required=True, default='HTTP:80', show_default=True,
              callback=protocol_add.parse_proto,
              help='PROTOCOL:PORT string for incoming internet connections.')
@click.option('--backend', '-b', required=True, default='HTTP:80', show_default=True,
              callback=protocol_add.parse_proto,
              help='PROTOCOL:PORT string for connecting to backend servers.')
@click.option('--method', '-m', help="Balancing Method.", default='ROUNDROBIN', show_default=True,
              type=click.Choice(['ROUNDROBIN', 'LEASTCONNECTION', 'WEIGHTED_RR']))
@click.option('--session', '-s', required=True,
              help="Session stickiness type. Valid values are SOURCE_IP or HTTP_COOKIE ")
@click.option('--max', help="Max Connections setting", type=int)
@environment.pass_env
def cli(env, identifier, **args):
    """Edit a load balancer protocol."""

    mgr = SoftLayer.LoadBalancerManager(env.client)

    uuid = mgr.get_lb_uuid(identifier)

    backend = args.get('backend')
    frontend = args.get('frontend')
    protocol_configurations = [
        {
            "backendPort": backend.get('port'),
            "backendProtocol": backend.get('protocol'),
            "frontendPort": frontend.get('port'),
            "frontendProtocol": frontend.get('protocol'),
            "loadBalancingMethod": args.get('method'),
            "sessionType": args.get('session'),
            "maxConn": args.get('max')
        }
    ]
    protocol_configurations[0]['listenerUuid'] = args.get('uuid')
    protocol = mgr.add_protocols(uuid, protocol_configurations)

    table = formatting.KeyValueTable(['name', 'value'])
    table.align['name'] = 'r'
    table.align['value'] = 'l'
    table.add_row(['Id', protocol.get('id')])
    table.add_row(['UUI', protocol.get('uuid')])
    table.add_row(['Address', protocol.get('address')])
    table.add_row(['Type', mgr.get_lb_type(protocol.get('type'))])
    table.add_row(['Description', protocol.get('description')])

    env.fout(table)



"""Create a zone."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('zone')
@environment.pass_env
def cli(env, zone):
    """Create a zone.

    Example::
        slcli dns zone-create ibm.com
        This command creates a zone that is named ibm.com.
"""

    manager = SoftLayer.DNSManager(env.client)
    manager.create_zone(zone)



"""Get details for an image."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers
from SoftLayer.CLI import image as image_mod
from SoftLayer import utils


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@environment.pass_env
def cli(env, identifier):
    """Get details for an image."""

    image_mgr = SoftLayer.ImageManager(env.client)
    image_id = helpers.resolve_id(image_mgr.resolve_ids, identifier, 'image')
    image = image_mgr.get_image(image_id, mask=image_mod.DETAIL_MASK)

    children_images = image.get('children')
    total_size = utils.lookup(image, 'firstChild', 'blockDevicesDiskSpaceTotal') or 0

    table = formatting.KeyValueTable(['name', 'value'])
    table.align['name'] = 'r'
    table.align['value'] = 'l'

    table.add_row(['Id', image['id']])
    table.add_row(['Global Identifier',
                   image.get('globalIdentifier', formatting.blank())])
    table.add_row(['Name', image['name'].strip()])
    table.add_row(['Status', formatting.FormattedItem(
        utils.lookup(image, 'status', 'keyname'),
        utils.lookup(image, 'status', 'name'),
    )])

    table.add_row([
        'Active Transaction',
        formatting.listing(_get_transaction_groups(children_images), separator=','),
    ])
    table.add_row(['Account', image.get('accountId', formatting.blank())])
    table.add_row(['Created', image.get('createDate')])
    table.add_row(['Total Size', formatting.b_to_gb(total_size)])
    table.add_row(['Visibility',
                   image_mod.PUBLIC_TYPE if image['publicFlag']
                   else image_mod.PRIVATE_TYPE])
    table.add_row(['Type',
                   formatting.FormattedItem(
                       utils.lookup(image, 'imageType', 'keyName'),
                       utils.lookup(image, 'imageType', 'name'),
                   )])
    table.add_row(['Flex', image.get('flexImageFlag')])
    table.add_row(['Note', image.get('note')])
    table.add_row(['Datacenters', _get_datacenter_table(children_images)])
    table.add_row(['Virtual Disks', _get_virtual_disks(children_images)])
    table.add_row(['Share Image', _get_share_image(image)])

    env.fout(table)


def _get_datacenter_table(children_images):
    """Returns image details as datacenter, size, and transaction within a formatting table.

      :param children_images: A list of images.
      """
    table_datacenter = formatting.Table(['DC', 'Size', 'Transaction'])
    for child in children_images:
        table_datacenter.add_row([
            utils.lookup(child, 'datacenter', 'name'),
            formatting.b_to_gb(child.get('blockDevicesDiskSpaceTotal', 0)),
            formatting.transaction_status(child.get('transaction'))
        ])

    return table_datacenter


def _get_transaction_groups(children_images):
    """Returns a Set of transaction groups.

      :param children_images: A list of images.
      """
    transactions = set()
    for child in children_images:
        transactions.add(utils.lookup(child, 'transaction', 'transactionGroup', 'name'))

    return transactions


def _get_virtual_disks(children_images):
    """Returns datacenters as a table.

      :param children_images: A list of images.
      """

    table_virtual_disks = formatting.Table(['Device', 'Capacity', 'Size On Disk'])

    if utils.lookup(children_images[0], 'blockDevices'):
        for block_devices in children_images[0]['blockDevices']:
            device_name = utils.lookup(block_devices, 'diskImage', 'name')
            software_references = utils.lookup(block_devices, 'diskImage', 'softwareReferences')
            if len(software_references) > 0:
                device_name = utils.lookup(software_references[0], 'softwareDescription', 'longDescription')

            size_on_disk = formatting.b_to_gb(block_devices.get('diskSpace', 0))
            if block_devices.get('diskSpace', 0) == 0:
                size_on_disk = 'N/A'

            capacity = str(utils.lookup(block_devices, 'diskImage', 'capacity')) + \
                ' ' + utils.lookup(block_devices, 'diskImage', 'units')

            table_virtual_disks.add_row([
                device_name,
                capacity,
                size_on_disk
            ])

    return table_virtual_disks


def _get_share_image(image):
    """Returns shares image as a table.

      :param image: Detail information about image.
      """
    table_share_account = formatting.Table(['Account', 'Shared On'])
    if utils.lookup(image, 'accountReferences'):
        for account in image['accountReferences']:
            if account['accountId'] != image['accountId']:
                table_share_account.add_row([
                    utils.lookup(account, 'accountId'),
                    utils.lookup(account, 'createDate'),
                ])

    return table_share_account



"""List VLANs."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.CLI.vlan.detail import get_gateway_firewall
from SoftLayer import utils

COLUMNS = [
    'Id',
    'Number',
    'Fully qualified name',
    'Name',
    'Network',
    'Data center',
    'Pod',
    'Gateway/Firewall',
    'Hardware',
    'Virtual servers',
    'Public ips',
    'Premium',
    'Tags'
]


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.option('--sortby', help='Column to sort by', type=click.Choice(COLUMNS))
@click.option('--datacenter', '-d',
              help='Filter by datacenter shortname (sng01, dal05, ...)')
@click.option('--number', '-n', help='Filter by VLAN number')
@click.option('--name', help='Filter by VLAN name')
@click.option('--limit', '-l', default=100, show_default=True,
              help='How many results to get in one api call, default is 100')
@environment.pass_env
def cli(env, sortby, datacenter, number, name, limit):
    """List VLANs.

    Note: A * Indicates a POD is closing soon. Ex:[red] Pod01* [/red]
    """

    mgr = SoftLayer.NetworkManager(env.client)

    table = formatting.Table(COLUMNS)
    table.sortby = sortby

    vlans = mgr.list_vlans(datacenter=datacenter, vlan_number=number, name=name, limit=limit)

    pod_mask = """mask[name, capabilities]"""
    pods = mgr.get_pods(mask=pod_mask)

    for vlan in vlans:
        billing = 'Yes' if vlan.get('billingItem') else 'No'

        table.add_row([
            vlan.get('id'),
            vlan.get('vlanNumber'),
            vlan.get('fullyQualifiedName'),
            vlan.get('name') or formatting.blank(),
            vlan.get('networkSpace', 'Direct Link').capitalize(),
            utils.lookup(vlan, 'datacenter', 'name'),
            get_pod_with_closed_announcement(vlan, pods),
            get_gateway_firewall(vlan),
            vlan.get('hardwareCount'),
            vlan.get('virtualGuestCount'),
            vlan.get('totalPrimaryIpAddressCount'),
            billing,
            formatting.tags(vlan['tagReferences'])
        ])

    env.fout(table)


def get_pod_with_closed_announcement(vlan, pods):
    """Gets pods with announcement to close"""
    for pod in pods:
        if utils.lookup(pod, 'name') == utils.lookup(vlan, 'podName'):
            if 'CLOSURE_ANNOUNCED' in utils.lookup(pod, 'capabilities'):
                name_pod = utils.lookup(pod, 'name').split('.')[1] + '*'
                return "[red]" + name_pod.capitalize() + "[/red]"
            else:
                return utils.lookup(pod, 'name').split('.')[1].capitalize()
    return ''



"""User details."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers
from SoftLayer import utils


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--keys', '-k', is_flag=True, default=False,
              help="Show the users API key.")
@click.option('--permissions', '-p', is_flag=True, default=False,
              help="Display permissions assigned to this user. Master users will show no permissions")
@click.option('--hardware', '-h', is_flag=True, default=False,
              help="Display hardware this user has access to.")
@click.option('--virtual', '-v', is_flag=True, default=False,
              help="Display virtual guests this user has access to.")
@click.option('--logins', '-l', is_flag=True, default=False,
              help="Show login history of this user for the last 30 days")
@click.option('--events', '-e', is_flag=True, default=False,
              help="Show event log for this user.")
@environment.pass_env
def cli(env, identifier, keys, permissions, hardware, virtual, logins, events):
    """User details."""

    mgr = SoftLayer.UserManager(env.client)
    user_id = helpers.resolve_id(mgr.resolve_ids, identifier, 'username')
    object_mask = "userStatus[name], parent[id, username], apiAuthenticationKeys[authenticationKey], "\
                  "unsuccessfulLogins, successfulLogins"

    user = mgr.get_user(user_id, object_mask)
    env.fout(basic_info(user, keys))

    if permissions:
        perms = mgr.get_user_permissions(user_id)
        env.fout(print_permissions(perms))
    if hardware:
        mask = "id, hardware, dedicatedHosts"
        access = mgr.get_user(user_id, mask)
        env.fout(print_dedicated_access(access.get('dedicatedHosts', [])))
        env.fout(print_access(access.get('hardware', []), 'Hardware'))
    if virtual:
        mask = "id, virtualGuests"
        access = mgr.get_user(user_id, mask)
        env.fout(print_access(access.get('virtualGuests', []), 'Virtual Guests'))
    if logins:
        login_log = mgr.get_logins(user_id)
        env.fout(print_logins(login_log))
    if events:
        event_log = mgr.get_events(user_id)
        env.fout(print_events(event_log))


def basic_info(user, keys):
    """Prints a table of basic user information"""

    table = formatting.KeyValueTable(['Title', 'Basic Information'])
    table.align['Title'] = 'r'
    table.align['Basic Information'] = 'l'

    table.add_row(['Id', user.get('id', '-')])
    table.add_row(['Username', user.get('username', '-')])
    keys_array = user.get('apiAuthenticationKeys')
    if keys and len(keys_array) != 0:
        for key in keys_array:
            table.add_row(['APIKEY', key.get('authenticationKey')])
    elif keys_array is not None and len(keys_array) != 0:
        table.add_row(['APIKEY', 'Yes'])
    else:
        table.add_row(['APIKEY', 'No'])
    table.add_row(['Name', f"{user.get('firstName', '-')} {user.get('lastName', '-')}"])
    table.add_row(['Email', user.get('email')])
    table.add_row(['OpenID', user.get('openIdConnectUserName')])
    address = f"{user.get('address1')} \
        {user.get('address2')} {user.get('city')} {user.get('state')} {user.get('country')} {user.get('postalCode')}"
    table.add_row(['Address', address])
    table.add_row(['Company', user.get('companyName')])
    table.add_row(['Created', user.get('createDate')])
    table.add_row(['Phone Number', user.get('officePhone')])
    if user.get('parentId', False):
        table.add_row(['Parent User', utils.lookup(user, 'parent', 'username')])
    table.add_row(
        ['Status', utils.lookup(user, 'userStatus', 'name')])
    table.add_row(['SSL VPN', user.get('sslVpnAllowedFlag', 'No')])
    for login in user.get('unsuccessfulLogins', {}):
        login_string = f"{login.get('createDate')} From: {login.get('ipAddress')}"
        table.add_row(['Last Failed Login', login_string])
        break
    for login in user.get('successfulLogins', {}):
        login_string = f"{login.get('createDate')} From: {login.get('ipAddress')}"
        table.add_row(['Last Login', login_string])
        break

    return table


def print_permissions(permissions):
    """Prints out a users permissions"""

    table = formatting.Table(['keyName', 'Description'])
    for perm in permissions:
        table.add_row([perm['keyName'], perm['name']])
    return table


def print_access(access, title):
    """Prints out the hardware or virtual guests a user can access"""

    columns = ['id', 'hostname', 'Primary Public IP', 'Primary Private IP', 'Created']
    table = formatting.Table(columns, title)

    for host in access:
        host_id = host.get('id')
        host_fqdn = host.get('fullyQualifiedDomainName', '-')
        host_primary = host.get('primaryIpAddress')
        host_private = host.get('primaryBackendIpAddress')
        host_created = host.get('provisionDate')
        table.add_row([host_id, host_fqdn, host_primary, host_private, host_created])
    return table


def print_dedicated_access(access):
    """Prints out the dedicated hosts a user can access"""

    table = formatting.Table(['id', 'Name', 'Cpus', 'Memory', 'Disk', 'Created'], 'Dedicated Access')
    for host in access:
        host_id = host.get('id')
        host_fqdn = host.get('name')
        host_cpu = host.get('cpuCount')
        host_mem = host.get('memoryCapacity')
        host_disk = host.get('diskCapacity')
        host_created = host.get('createDate')
        table.add_row([host_id, host_fqdn, host_cpu, host_mem, host_disk, host_created])
    return table


def print_logins(logins):
    """Prints out the login history for a user"""
    table = formatting.Table(['Date', 'IP Address', 'Successufl Login?'])
    for login in logins:
        table.add_row([login.get('createDate'), login.get('ipAddress'), login.get('successFlag')])
    return table


def print_events(events):
    """Prints out the event log for a user"""
    columns = ['Date', 'Type', 'IP Address', 'label', 'username']
    table = formatting.Table(columns)
    for event in events:
        table.add_row([event.get('eventCreateDate'), event.get('eventName'),
                       event.get('ipAddress'), event.get('label'), event.get('username')])
    return table



"""List active Load Balancer as a Service devices."""
import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer import utils


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@environment.pass_env
def cli(env):
    """List active Load Balancer as a Service devices."""
    mgr = SoftLayer.LoadBalancerManager(env.client)

    lbaas = mgr.get_lbaas()
    if lbaas:
        lbaas_table = generate_lbaas_table(lbaas)
        env.fout(lbaas_table)

    else:
        env.fout("No LBaaS devices found")


def location_sort(location):
    """Quick function that just returns the datacenter longName for sorting"""
    return utils.lookup(location, 'datacenter', 'longName')


def generate_lbaas_table(lbaas):
    """Takes a list of SoftLayer_Network_LBaaS_LoadBalancer and makes a table"""
    table = formatting.Table([
        'Id', 'Location', 'Name', 'Description', 'Public', 'Create Date', 'Members', 'Listeners'
    ], title="IBM Cloud LoadBalancer")

    table.align['Name'] = 'l'
    table.align['Description'] = 'l'
    table.align['Location'] = 'l'
    for this_lb in sorted(lbaas, key=location_sort):
        table.add_row([
            this_lb.get('id'),
            utils.lookup(this_lb, 'datacenter', 'longName'),
            this_lb.get('name'),
            this_lb.get('description'),
            'Yes' if this_lb.get('isPublic', 1) == 1 else 'No',
            utils.clean_time(this_lb.get('createDate')),
            this_lb.get('memberCount', 0),
            this_lb.get('listenerCount', 0)


        ])
    return table



"""List options for creating Placement Groups"""
# :license: MIT, see LICENSE for more details.

import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.managers.vs_placement import PlacementManager as PlacementManager


@click.command(cls=SLCommand)
@environment.pass_env
def cli(env):
    """List options for creating a placement group."""
    manager = PlacementManager(env.client)

    routers = manager.get_routers()
    env.fout(get_router_table(routers))

    rules = manager.get_all_rules()
    env.fout(get_rule_table(rules))


def get_router_table(routers):
    """Formats output from _get_routers and returns a table. """
    table = formatting.Table(['Datacenter', 'Hostname', 'Backend Router Id'], "Available Routers")
    for router in routers:
        datacenter = router['topLevelLocation']['longName']
        table.add_row([datacenter, router['hostname'], router['id']])
    return table


def get_rule_table(rules):
    """Formats output from get_all_rules and returns a table. """
    table = formatting.Table(['Id', 'KeyName'], "Rules")
    for rule in rules:
        table.add_row([rule['id'], rule['keyName']])
    return table



getPrivateBlockDeviceTemplateGroups = [{
    'accountId': 1234,
    'blockDevices': [],
    'createDate': '2013-12-05T21:53:03-06:00',
    'globalIdentifier': 'E6DBD73B-1651-4B28-BCBA-A11DF7C9D79E',
    'id': 200,
    'name': 'test_image',
    'parentId': '',
    'publicFlag': False,
}, {
    'accountId': 1234,
    'blockDevices': [],
    'createDate': '2013-12-05T21:53:03-06:00',
    'globalIdentifier': 'F9329795-4220-4B0A-B970-C86B950667FA',
    'id': 201,
    # 'name': 'private_image2',
    'name': 'a¬ሴ€耀',
    'parentId': '',
    'publicFlag': False,
}]

getVirtualGuests = [{
    'id': 100,
    'metricTrackingObjectId': 1,
    'hostname': 'vs-test1',
    'domain': 'test.sftlyr.ws',
    'fullyQualifiedDomainName': 'vs-test1.test.sftlyr.ws',
    'status': {'keyName': 'ACTIVE', 'name': 'Active'},
    'datacenter': {'id': 50, 'name': 'TEST00',
                   'description': 'Test Data Center'},
    'powerState': {'keyName': 'RUNNING', 'name': 'Running'},
    'maxCpu': 2,
    'maxMemory': 1024,
    'primaryIpAddress': '172.16.240.2',
    'globalIdentifier': '1a2b3c-1701',
    'primaryBackendIpAddress': '10.45.19.37',
    'hourlyBillingFlag': False,
    'pendingMigrationFlag': True,
    'billingItem': {
        'id': 6327,
        'recurringFee': 1.54,
        'orderItem': {
            'order': {
                'userRecord': {
                    'username': 'chechu',
                }
            }
        }
    },
}, {
    'id': 104,
    'metricTrackingObjectId': 2,
    'hostname': 'vs-test2',
    'domain': 'test.sftlyr.ws',
    'fullyQualifiedDomainName': 'vs-test2.test.sftlyr.ws',
    'status': {'keyName': 'ACTIVE', 'name': 'Active'},
    'datacenter': {'id': 50, 'name': 'TEST00',
                   'description': 'Test Data Center'},
    'powerState': {'keyName': 'RUNNING', 'name': 'Running'},
    'maxCpu': 4,
    'maxMemory': 4096,
    'primaryIpAddress': '172.16.240.7',
    'globalIdentifier': '05a8ac-6abf0',
    'primaryBackendIpAddress': '10.45.19.35',
    'hourlyBillingFlag': True,
    'pendingMigrationFlag': True,
    'billingItem': {
        'id': 6327,
        'recurringFee': 1.54,
        'orderItem': {
            'order': {
                'userRecord': {
                    'username': 'chechu',
                }
            }
        }
    },
    'virtualRack': {'id': 1, 'bandwidthAllotmentTypeId': 2},
}]

getMonthlyVirtualGuests = [vs for vs in getVirtualGuests
                           if not vs['hourlyBillingFlag']]
getHourlyVirtualGuests = [vs for vs in getVirtualGuests
                          if vs['hourlyBillingFlag']]

getHardware = [{
    'id': 1000,
    'metricTrackingObject': {'id': 3},
    'globalIdentifier': '1a2b3c-1701',
    'datacenter': {'id': 50, 'name': 'TEST00',
                   'description': 'Test Data Center'},
    'billingItem': {
        'id': 6327,
        'recurringFee': 1.54,
        'orderItem': {
            'order': {
                'userRecord': {
                    'username': 'chechu',
                }
            }
        }
    },
    'primaryIpAddress': '172.16.1.100',
    'hostname': 'hardware-test1',
    'domain': 'test.sftlyr.ws',
    'fullyQualifiedDomainName': 'hardware-test1.test.sftlyr.ws',
    'processorPhysicalCoreAmount': 2,
    'memoryCapacity': 2,
    'primaryBackendIpAddress': '10.1.0.2',
    'networkManagementIpAddress': '10.1.0.3',
    'hardwareStatus': {'status': 'ACTIVE'},
    'provisionDate': '2013-08-01 15:23:45',
    'notes': 'These are test notes.',
    'operatingSystem': {
        'softwareLicense': {
            'softwareDescription': {
                'referenceCode': 'Ubuntu',
                'name': 'Ubuntu 12.04 LTS',
            }
        },
        'passwords': [
            {'username': 'root', 'password': 'abc123'}
        ],
    },
    'networkVlans': [
        {
            'networkSpace': 'PRIVATE',
            'vlanNumber': 1800,
            'id': 9653
        },
        {
            'networkSpace': 'PUBLIC',
            'vlanNumber': 3672,
            'id': 19082
        },
    ],
    'tagReferences': [
        {'tag': {'name': 'test_tag'}}
    ],
    'activeTransaction': {
        'transactionStatus': {
            'name': 'TXN_NAME',
            'friendlyName': 'Friendly Transaction Name',
            'id': 6660
        }
    },
    "virtualHost": {
        "accountId": 11111,
        "id": 22222,
        "name": "vmware.chechu.com",
        "guests": [
            {
                "accountId": 11111,
                "createDate": "2019-09-05T17:03:42-06:00",
                "hostname": "NSX-T Manager",
                "id": 33333,
                "maxCpu": 16,
                "maxCpuUnits": "CORE",
                "maxMemory": 49152,
                "powerState": {
                    "keyName": "RUNNING",
                    "name": "Running"
                },
                "status": {
                    "keyName": "ACTIVE",
                    "name": "Active"
                }
            }]}
}, {
    'id': 1001,
    'metricTrackingObject': {'id': 4},
    'globalIdentifier': '1a2b3c-1702',
    'datacenter': {'name': 'TEST00',
                   'description': 'Test Data Center'},
    'billingItem': {
        'id': 7112,
        'orderItem': {
            'order': {
                'userRecord': {
                    'username': 'chechu',
                }
            }
        }
    },
    'primaryIpAddress': '172.16.4.94',
    'hostname': 'hardware-test2',
    'domain': 'test.sftlyr.ws',
    'fullyQualifiedDomainName': 'hardware-test2.test.sftlyr.ws',
    'processorPhysicalCoreAmount': 4,
    'memoryCapacity': 4,
    'primaryBackendIpAddress': '10.1.0.3',
    'hardwareStatus': {'status': 'ACTIVE'},
    'provisionDate': '2013-08-03 07:15:22',
    'operatingSystem': {
        'softwareLicense': {
            'softwareDescription': {
                'referenceCode': 'Ubuntu',
                'name': 'Ubuntu 12.04 LTS',
            }
        }
    },
    'networkVlans': [
        {
            'networkSpace': 'PRIVATE',
            'vlanNumber': 1800,
            'id': 9653
        },
        {
            'networkSpace': 'PUBLIC',
            'vlanNumber': 3672,
            'id': 19082
        },
    ],
    "virtualHost": {
        "accountId": 11111,
        "id": 22222,
        "name": "host14.vmware.chechu.com",
        "guests": []
    }
}, {
    'id': 1002,
    'metricTrackingObject': {'id': 5},
    'datacenter': {'name': 'TEST00',
                   'description': 'Test Data Center'},
    'billingItem': {
        'id': 7112,
        'orderItem': {
            'order': {
                'userRecord': {
                    'username': 'chechu',
                }
            }
        }
    },
    'primaryIpAddress': '172.16.4.95',
    'hostname': 'hardware-bad-memory',
    'domain': 'test.sftlyr.ws',
    'fullyQualifiedDomainName': 'hardware-bad-memory.test.sftlyr.ws',
    'processorPhysicalCoreAmount': 4,
    'memoryCapacity': None,
    'primaryBackendIpAddress': '10.1.0.4',
    'hardwareStatus': {'status': 'ACTIVE'},
    'provisionDate': '2014-04-02 13:48:00',
    'operatingSystem': {
        'softwareLicense': {
            'softwareDescription': {
                'referenceCode': 'Ubuntu',
                'name': 'Ubuntu 12.04 LTS',
            }
        }
    },
    'networkVlans': [
        {
            'networkSpace': 'PRIVATE',
            'vlanNumber': 1800,
            'id': 9653
        },
        {
            'networkSpace': 'PUBLIC',
            'vlanNumber': 3672,
            'id': 19082
        },
    ],
    "virtualHost": {
        "accountId": 11111,
        "id": 22222,
        "name": "host14.vmware.chechu.com",
        "guests": []
    }
}, {
    'id': 1003,
    "virtualHost": {
        "accountId": 11111,
        "id": 22222,
        "name": "host14.vmware.chechu.com",
        "guests": []
    }
}]
getDomains = [{'name': 'example.com',
               'id': 12345,
               'serial': 2014030728,
               'updateDate': '2014-03-07T13:52:31-06:00'}]

getObject = {
    'cdnAccounts': [
        {
            "cdnAccountName": "1234a",
            "providerPortalAccessFlag": False,
            "createDate": "2012-06-25T14:05:28-07:00",
            "id": 1234,
            "legacyCdnFlag": False,
            "dependantServiceFlag": True,
            "cdnSolutionName": "ORIGIN_PULL",
            "statusId": 4,
            "accountId": 1234
        },
        {
            "cdnAccountName": "1234a",
            "providerPortalAccessFlag": False,
            "createDate": "2012-07-24T13:34:25-07:00",
            "id": 1234,
            "legacyCdnFlag": False,
            "dependantServiceFlag": False,
            "cdnSolutionName": "POP_PULL",
            "statusId": 4,
            "accountId": 1234
        }
    ],
    'accountId': 1234
}

getRwhoisData = {
    'abuseEmail': 'abuseEmail',
    'accountId': 1234,
    'address1': 'address1',
    'address2': 'address2',
    'city': 'city',
    'companyName': 'companyName',
    'country': 'country',
    'createDate': 'createDate',
    'firstName': 'firstName',
    'id': 'id',
    'lastName': 'lastName',
    'modifyDate': 'modifyDate',
    'postalCode': 'postalCode',
    'privateResidenceFlag': True,
}

getGlobalIpRecords = [{
    'id': '200',
    'ipAddress': {
        'subnet': {
            'networkIdentifier': '10.0.0.1',
        },
        'ipAddress': '127.0.0.1',
    },
    'destinationIpAddress': {
        'ipAddress': '127.0.0.1',
        'virtualGuest': {'fullyQualifiedDomainName': 'example.com'}}
}, {
    'id': '201',
    'ipAddress': {
        'subnet': {
            'networkIdentifier': '10.0.0.1',
        },
        'ipAddress': '127.0.0.1',
    },
    'destinationIpAddress': {
        'ipAddress': '127.0.0.1',
        'hardware': {'fullyQualifiedDomainName': 'example.com'}}
}]

getSubnets = [
    {
        'id': '100',
        'networkIdentifier': '10.0.0.1',
        'cidr': '/24',
        'networkVlanId': 123,
        'datacenter': {'name': 'dal00'},
        'version': 4,
        'subnetType': 'PRIMARY',
        'ipAddressCount': 10,
        'virtualGuests': [],
        'hardware': [],
        "podName": "dal05.pod04",
        "networkVlan": {
            "accountId": 123,
            "id": 2581232,
            "modifyDate": "2019-07-17T01:09:51+08:00",
            "vlanNumber": 795
        }
    },
    {
        "gateway": "5.111.11.111",
        "id": '111',
        "modifyDate": "2018-07-24T17:14:57+08:00",
        'networkIdentifier': '10.0.0.1',
        'ipAddressCount': 10,
        'cidr': '/24',
        'virtualGuests': [],
        'hardware': [],
        "networkVlanId": 22222,
        "sortOrder": "2",
        "subnetType": "SECONDARY_ON_VLAN",
        "totalIpAddresses": "8",
        "usableIpAddressCount": "5",
        "version": 4
    }
]

getSshKeys = [{'id': '100', 'label': 'Test 1'},
              {'id': '101', 'label': 'Test 2',
               'finterprint': 'aa:bb:cc:dd',
               'notes': 'my key'}]

getSecurityCertificates = [{'certificate': '1234',
                            'commonName': 'cert',
                            'id': 1234,
                            'validityDays': 0, }]
getExpiredSecurityCertificates = getSecurityCertificates
getValidSecurityCertificates = getSecurityCertificates

getTickets = [
    {
        "accountId": 1234,
        "serviceProviderResourceId": "CS123456",
        "assignedUserId": 12345,
        "createDate": "2013-08-01T14:14:04-07:00",
        "id": 100,
        "lastEditDate": "2013-08-01T14:16:47-07:00",
        "lastEditType": "AUTO",
        "modifyDate": "2013-08-01T14:16:47-07:00",
        "status": {
            "id": 1002,
            "name": "Closed"
        },
        "statusId": 1002,
        "title": "Cloud Instance Cancellation - 08/01/13"
    },
    {
        "accountId": 1234,
        "serviceProviderResourceId": "CS123456",
        "assignedUserId": 12345,
        "createDate": "2013-08-01T14:14:04-07:00",
        "id": 101,
        "lastEditDate": "2013-08-01T14:16:47-07:00",
        "lastEditType": "AUTO",
        "modifyDate": "2013-08-01T14:16:47-07:00",
        "status": {
            "id": 1002,
            "name": "Closed"
        },
        "statusId": 1002,
        "title": "Cloud Instance Cancellation - 08/01/13"
    },
    {
        "accountId": 1234,
        "serviceProviderResourceId": "CS123456",
        "assignedUserId": 12345,
        "createDate": "2014-03-03T09:44:01-08:00",
        "id": 102,
        "lastEditDate": "2013-08-01T14:16:47-07:00",
        "lastEditType": "AUTO",
        "modifyDate": "2014-03-03T09:44:03-08:00",
        "status": {
            "id": 1001,
            "name": "Open"
        },
        'assignedUser': {'firstName': 'John', 'lastName': 'Smith'},
        "statusId": 1001,
        "title": "Cloud Instance Cancellation - 08/01/13"
    }]

getOpenTickets = [ticket for ticket in getTickets
                  if ticket['statusId'] == 1001]
getClosedTickets = [ticket for ticket in getTickets
                    if ticket['statusId'] == 1002]

getCurrentUser = {'id': 12345, 'username': 'testAccount',
                  'apiAuthenticationKeys': [{'authenticationKey': 'A' * 64}],
                  'accountId': 123456789}

getCdnAccounts = [
    {
        "cdnAccountName": "1234a",
        "providerPortalAccessFlag": False,
        "createDate": "2012-06-25T14:05:28-07:00",
        "id": 1234,
        "legacyCdnFlag": False,
        "dependantServiceFlag": True,
        "cdnSolutionName": "ORIGIN_PULL",
        "statusId": 4,
        "accountId": 1234
    },
    {
        "cdnAccountName": "1234a",
        "providerPortalAccessFlag": False,
        "createDate": "2012-07-24T13:34:25-07:00",
        "id": 1234,
        "legacyCdnFlag": False,
        "dependantServiceFlag": False,
        "cdnSolutionName": "POP_PULL",
        "statusId": 4,
        "accountId": 1234
    }
]

getNetworkVlans = [{
    'id': 1,
    'networkSpace': '',
    'hardwareCount': 0,
    'hardware': [],
    'vlanNumber': 12,
    'networkComponents': [],
    'primaryRouter': {
        'datacenter': {'name': 'dal00'}
    },
    'virtualGuestCount': 0,
    'virtualGuests': [],
    'dedicatedFirewallFlag': True,
    'highAvailabilityFirewallFlag': True,
    'networkVlanFirewall': {'id': 1234},
    'totalPrimaryIpAddressCount': 1,
    'subnetCount': 0,
    'subnets': [],
    'tagReferences': [],
    'firewallInterfaces': [
        {
            'id': 1,
            'name': 'outside'
        },
        {
            'id': 12,
            'name': 'inside'
        }
    ]
}, {
    'id': 2,
    'networkSpace': 'PRIVATE',
    'totalPrimaryIpAddressCount': 2,
    'dedicatedFirewallFlag': False,
    'highAvailabilityFirewallFlag': True,
    'networkVlanFirewall': {'id': 7896},
    'hardwareCount': 0,
    'hardware': [],
    'vlanNumber': 13,
    'networkComponents': [],
    'primaryRouter': {
        'datacenter': {'name': 'dal00'}
    },
    'virtualGuestCount': 0,
    'virtualGuests': [],
    'firewallGuestNetworkComponents': [{
        'id': 1234,
        'guestNetworkComponent': {'guest': {'id': 1}},
        'status': 'ok'}],
    'firewallNetworkComponents': [{
        'id': 1234,
        'networkComponent': {'downlinkComponent': {'hardwareId': 1}},
        'status': 'ok'}],
    'firewallInterfaces': [],

    'subnetCount': 0,
    'tagReferences': [],
    'subnets': [],
}, {
    'id': 3,
    'networkSpace': 'PRIVATE',
    'name': 'dal00',
    'hardwareCount': 1,
    'dedicatedFirewallFlag': True,
    'highAvailabilityFirewallFlag': True,
    'networkVlanFirewall': {'id': 23456},
    'vlanNumber': 14,
    'hardware': [{'id': 1}],
    'networkComponents': [{'id': 2}],
    'primaryRouter': {
        'datacenter': {'name': 'dal00'}
    },
    'firewallInterfaces': [
        {
            'id': 31,
            'name': 'outside'
        }],
    'totalPrimaryIpAddressCount': 3,
    'subnetCount': 0,
    'subnets': [],
    'tagReferences': [],
    'virtualGuestCount': 1,
    'virtualGuests': [{'id': 3}]
}]

getAdcLoadBalancers = []

getNasNetworkStorage = [{
    'id': 1,
    'capacityGb': 10,
    'serviceResource': {'datacenter': {'name': 'Dallas'}},
    'username': 'user',
    'password': 'pass',
    'serviceResourceBackendIpAddress': '127.0.0.1',
    'storageType': {'keyName': 'ENDURANCE_STORAGE'},
    'fileNetworkMountAddress': '127.0.0.1:/TEST',
}]

getActiveQuotes = [{
    'accountId': 1234,
    'id': 1234,
    'name': 'TestQuote1234',
    'quoteKey': '1234test4321',
    'createDate': '2019-04-10T14:26:03-06:00',
    'modifyDate': '2019-04-10T14:26:03-06:00',
    'order': {
        'id': 37623333,
        'items': [
            {
                'categoryCode': 'guest_core',
                'description': '4 x 2.0 GHz or higher Cores',
                'id': 468394713,
                'itemId': 859,
                'itemPriceId': '1642',
                'oneTimeAfterTaxAmount': '0',
                'oneTimeFee': '0',
                'oneTimeFeeTaxRate': '0',
                'oneTimeTaxAmount': '0',
                'quantity': 1,
                'recurringAfterTaxAmount': '0',
                'recurringFee': '0',
                'recurringTaxAmount': '0',
                'setupAfterTaxAmount': '0',
                'setupFee': '0',
                'setupFeeDeferralMonths': None,
                'setupFeeTaxRate': '0',
                'setupTaxAmount': '0',
                'package': {'id': 46, 'keyName': 'CLOUD_SERVER'}
            },
        ]
    }
}]

getOrders = [{
    'id': 1234,
    'resourceType': '1 x 2.0 GHz Core',
    'hostName': 'test',
    'createDate': '2014-05-01T14:03:04-07:00',
    'cost': 0.0
}]

getBillingInfo = [{
    'id': 1234,
    'accountId': 123,
    'resourceType': '1 x 2.0 GHz Core',
    'hostName': 'test',
    'modifyDate': '2014-05-01T14:03:04-07:00',
    'createDate': '2014-05-01T14:03:04-07:00',
    'anniversaryDayOfMonth': 2,
    'percentDiscountOnetime': 2,
    'sparePoolAmount': 0,
    'currency': {
        'KeyName': 'usd',
        'id': 1,
        'name': 'dollars'
    }
}]

getLatestBillDate = '2014-05-01T14:03:04-07:00'

getBalance = 40

getNextInvoiceTotalAmount = 2

getHubNetworkStorage = [
    {
        "id": 12345,
        "username": "SLOS12345-1",
        "billingItem": {
            "createDate": "2020-10-08T11:45:17-06:00"
        },
        "serviceResource": {
            "name": "Cleversafe - US Region"
        },
        "storageType": {
            "description": "Object Storage Standard Account",
            "id": 24,
            "keyName": "OBJECT_STORAGE_STANDARD"
        }
    },
    {
        "id": 12346,
        "username": "SLOS12345-2",
        "vendorName": "Swift",
        "billingItem": {
            "createDate": "2020-10-08T11:45:17-06:00"
        },
        "storageType": {
            "description": "Object Storage Standard Account",
            "id": 24,
            "keyName": "OBJECT_STORAGE_STANDARD"
        }
    }
]

getIscsiNetworkStorage = [{
    'accountId': 1234,
    'billingItem': {'id': 449},
    'capacityGb': 20,
    'createDate': '2015:50:15-04:00',
    'guestId': '',
    'hardwareId': '',
    'hostId': '',
    'id': 100,
    'nasType': 'ISCSI',
    'notes': """{'status': 'available'}""",
    'password': '',
    'serviceProviderId': 1,
    'serviceResource': {'datacenter': {'name': 'dal05', 'id': 449500}},
    'serviceResourceBackendIpAddress': '10.1.2.3',
    'serviceResourceName': 'Storage Type 01 Aggregate staaspar0101_pc01',
    'username': 'username',
    'storageType': {'keyName': 'ENDURANCE_STORAGE'},
}]

getVirtualDedicatedRacks = [{
    'id': 1,
    'name': 'my first pool',
    'metricTrackingObjectId': 10,
}]

getDedicatedHosts = [
    {
        "cpuCount": 56,
        "createDate": "2021-11-18T15:13:57-06:00",
        "diskCapacity": 1200,
        "id": 656700,
        "memoryCapacity": 242,
        "name": "dedicatedhost01",
        "guestCount": 0,
        "allocationStatus": {
            "cpuAllocated": 0,
            "cpuAvailable": 56,
            "cpuCount": 56,
            "diskAllocated": 0,
            "diskAvailable": 1200,
            "diskCapacity": 1200,
            "guestCount": 0,
            "memoryAllocated": 0,
            "memoryAvailable": 242,
            "memoryCapacity": 242
        },
        "backendRouter": {
            "accountId": 1,
            "bareMetalInstanceFlag": 0,
            "domain": "softlayer.com",
            "fullyQualifiedDomainName": "bcr01a.dal13.softlayer.com",
            "hardwareStatusId": 5,
            "hostname": "bcr01a.dal13",
            "id": 1883692,
            "notes": "",
            "provisionDate": None,
            "serviceProviderId": 1,
            "serviceProviderResourceId": None
        },
        "billingItem": {
            "allowCancellationFlag": 1,
            "cancellationDate": None,
            "categoryCode": "dedicated_virtual_hosts",
            "createDate": "2021-11-18T15:13:59-06:00",
            "currentHourlyCharge": "0",
            "cycleStartDate": "2023-01-04T00:07:36-06:00",
            "description": "56 Cores X 242 RAM X 1.2 TB",
            "hostName": "dedicatedhost01",
            "hourlyRecurringFee": "0",
            "hoursUsed": "465",
            "id": 910104016,
            "laborFee": "0",
            "laborFeeTaxRate": "0",
            "lastBillDate": "2023-01-04T00:07:36-06:00",
            "modifyDate": "2023-01-04T00:07:36-06:00",
            "nextBillDate": "2023-02-04T00:00:00-06:00",
            "oneTimeFee": "0",
            "oneTimeFeeTaxRate": "0",
            "orderItemId": 874370304,
            "parentId": None,
            "recurringFee": "0",
            "recurringFeeTaxRate": "0",
            "recurringMonths": 1,
            "serviceProviderId": 1,
            "setupFee": "0",
            "setupFeeTaxRate": "0",
            "orderItem": {
                "categoryCode": "dedicated_virtual_hosts",
                "description": "56 Cores X 242 RAM X 1.2 TB",
                "hostName": "dedicatedhost01",
                "hourlyRecurringFee": "0",
                "id": 874370304,
                "itemId": 10195,
                "itemPriceId": "200269",
                "laborAfterTaxAmount": "0",
                "laborFee": "0",
                "laborFeeTaxRate": "0",
                "laborTaxAmount": "0",
                "oneTimeAfterTaxAmount": "0",
                "oneTimeFee": "0",
                "oneTimeFeeTaxRate": "0",
                "oneTimeTaxAmount": "0",
                "parentId": None,
                "presetId": None,
                "promoCodeId": None,
                "quantity": None,
                "recurringAfterTaxAmount": "0",
                "recurringFee": "0",
                "recurringTaxAmount": "0",
                "setupAfterTaxAmount": "0",
                "setupFee": "0",
                "setupFeeDeferralMonths": None,
                "setupFeeTaxRate": "0",
                "setupTaxAmount": "0",
                "order": {
                    "accountId": 307608,
                    "createDate": "2021-11-18T15:13:57-06:00",
                    "id": 85857762,
                    "impersonatingUserRecordId": None,
                    "modifyDate": "2021-11-18T15:14:00-06:00",
                    "orderQuoteId": None,
                    "orderTypeId": 11,
                    "presaleEventId": None,
                    "privateCloudOrderFlag": False,
                    "status": "APPROVED",
                    "userRecordId": 7650493,
                    "userRecord": {
                        "accountId": 307608,
                        "address1": "4849 Alpha Rd",
                        "city": "Dallas",
                        "companyName": "SoftLayer Internal - Development Community",
                        "country": "US",
                        "createDate": "2019-06-10T15:21:33-07:00",
                        "daylightSavingsTimeFlag": False,
                        "denyAllResourceAccessOnCreateFlag": False,
                        "displayName": "danielc",
                        "email": "daniel.cabero@jalasoft.com",
                        "firstName": "daniel",
                        "forumPasswordHash": "unused",
                        "iamAuthorizationStatus": 1,
                        "iamId": "IBMid-550003CUGT",
                        "id": 7650493,
                        "isMasterUserFlag": False,
                        "lastName": "cabero",
                        "localeId": 1,
                        "managedByFederationFlag": False,
                        "managedByOpenIdConnectFlag": True,
                        "minimumPasswordLifeHours": 0,
                        "modifyDate": "2019-12-13T07:29:07-06:00",
                        "openIdConnectUserName": "daniel.cabero@jalasoft.com",
                        "parentId": 167758,
                        "passwordExpireDate": None,
                        "permissionCheckLikeMasterUserFlag": 0,
                        "postalCode": "75244-4608",
                        "pptpVpnAllowedFlag": False,
                        "preventPreviousPasswords": 0,
                        "savedId": "7650493",
                        "secondaryLoginManagementFlag": True,
                        "secondaryLoginRequiredFlag": None,
                        "secondaryPasswordModifyDate": "2019-06-10T15:21:33-07:00",
                        "secondaryPasswordTimeoutDays": 0,
                        "sslVpnAllowedFlag": False,
                        "state": "TX",
                        "statusDate": None,
                        "timezoneId": 113,
                        "userStatusId": 1001,
                        "username": "sl307608-dcabero",
                        "vpnManualConfig": False,
                        "ibmIdLink": {
                            "createDate": "2019-06-10T15:21:34-07:00",
                            "defaultFlag": 1,
                            "destinationUserAlphanumericId": "daniel%2ecabero@jalasoft%2ecom",
                            "destinationUserId": None,
                            "iamIdVerificationFlag": 2,
                            "id": 1051135,
                            "realm": "IBMid",
                            "serviceProviderId": 348,
                            "uniqueIdentifier": "IBMid-550003CUGT",
                            "userId": 7650493
                        },
                        "locale": {
                            "friendlyName": "English",
                            "id": 1,
                            "languageTag": "en-US",
                            "name": "English"
                        },
                        "timezone": {
                            "id": 113,
                            "longName": "(GMT-06:00) America/Chicago - CST",
                            "name": "America/Chicago",
                            "offset": "-0600",
                            "shortName": "CST"
                        },
                        "userStatus": {
                            "id": 1001,
                            "keyName": "ACTIVE",
                            "name": "Active"
                        }
                    }
                }
            },
            "resourceTableId": 656700
        },
        "datacenter": {
            "id": 1854895,
            "longName": "Dallas 13",
            "name": "dal13",
            "statusId": 2
        }
    }
]

getUsers = [
    {'displayName': 'ChristopherG',
     'hardwareCount': 138,
     'id': 11100,
     'userStatus': {'name': 'Active'},
     'username': 'SL1234',
     'virtualGuestCount': 99,
     'externalBindingCount': 1,
     'apiAuthenticationKeyCount': 1,
     },
    {'displayName': 'PulseL',
     'hardwareCount': 100,
     'id': 11111,
     'userStatus': {'name': 'Active'},
     'username': 'sl1234-abob',
     'virtualGuestCount': 99,
     }
]

getReservedCapacityGroups = [
    {
        'accountId': 1234,
        'backendRouterId': 1411193,
        'createDate': '2018-09-24T16:33:09-06:00',
        'id': 3103,
        'modifyDate': '',
        'name': 'test-capacity',
        'availableInstanceCount': 1,
        'instanceCount': 3,
        'occupiedInstanceCount': 1,
        'backendRouter': {
            'accountId': 1,
            'bareMetalInstanceFlag': 0,
            'domain': 'softlayer.com',
            'fullyQualifiedDomainName': 'bcr02a.dal13.softlayer.com',
            'hardwareStatusId': 5,
            'hostname': 'bcr02a.dal13',
            'id': 1411193,
            'notes': '',
            'provisionDate': '',
            'serviceProviderId': 1,
            'serviceProviderResourceId': '',
            'primaryIpAddress': '10.0.144.28',
            'datacenter': {
                'id': 1854895,
                'longName': 'Dallas 13',
                'name': 'dal13',
                'statusId': 2
            },
            'hardwareFunction': {
                'code': 'ROUTER',
                'description': 'Router',
                'id': 1
            },
            'topLevelLocation': {
                'id': 1854895,
                'longName': 'Dallas 13',
                'name': 'dal13',
                'statusId': 2
            }
        },
        'instances': [
            {
                'id': 3501,
                'billingItem': {
                    'description': 'B1.1x2 (1 Year Term)',
                    'hourlyRecurringFee': '.032'
                }
            },
            {
                'id': 3519,
                'billingItem': {
                    'description': 'B1.1x2 (1 Year Term)',
                    'hourlyRecurringFee': '.032'
                }
            },
            {
                'id': 3519
            }
        ]
    }
]

getPlacementGroups = [{
    "createDate": "2019-01-18T16:08:44-06:00",
    "id": 12345,
    "name": "test01",
    "guestCount": 0,
    "backendRouter": {
        "hostname": "bcr01a.mex01",
        "id": 329266
    },
    "rule": {
        "id": 1,
        "keyName": "SPREAD",
        "name": "SPREAD"
    }
}]

getInvoices = [
    {
        'id': 33816665,
        'modifyDate': '2019-03-04T00:17:42-06:00',
        'createDate': '2019-03-04T00:17:42-06:00',
        'startingBalance': '129251.73',
        'statusCode': 'OPEN',
        'typeCode': 'RECURRING',
        'itemCount': 3317,
        'invoiceTotalAmount': '6230.66'
    },
    {
        'id': 12345667,
        'modifyDate': '2019-03-05T00:17:42-06:00',
        'createDate': '2019-03-04T00:17:42-06:00',
        'startingBalance': '129251.73',
        'statusCode': 'OPEN',
        'typeCode': 'RECURRING',
        'itemCount': 12,
        'invoiceTotalAmount': '6230.66',
        'endingBalance': '12345.55'
    }
]

getApplicationDeliveryControllers = [
    {
        'accountId': 307608,
        'createDate': '2015-05-05T16:23:52-06:00',
        'id': 11449,
        'modifyDate': '2015-05-05T16:24:09-06:00',
        'name': 'TEST307608-1',
        'typeId': 2,
        'description': 'Citrix NetScaler VPX 10.5 10Mbps Standard',
        'managementIpAddress': '10.11.11.112',
        'outboundPublicBandwidthUsage': '.00365',
        'primaryIpAddress': '19.4.24.16',
        'datacenter': {
            'longName': 'Dallas 9',
            'name': 'dal09',
        },
        'password': {
            'password': 'aaaaa',
            'username': 'root'
        },
        'type': {
            'keyName': 'NETSCALER_VPX',
            'name': 'NetScaler VPX'
        }
    }
]

getScaleGroups = [
    {
        "accountId": 31111,
        "cooldown": 1800,
        "createDate": "2016-10-25T01:48:34+08:00",
        "id": 12222222,
        "lastActionDate": "2016-10-25T01:48:34+08:00",
        "maximumMemberCount": 5,
        "minimumMemberCount": 0,
        "name": "tests",
        "regionalGroupId": 663,
        "virtualGuestMemberTemplate": {
            "accountId": 31111,
            "domain": "sodg.com",
            "hostname": "testing",
            "id": None,
            "maxCpu": None,
            "maxMemory": 32768,
            "startCpus": 32,
            "blockDevices": [
                {
                    "device": "0",
                    "diskImage": {
                        "capacity": 25,
                    }
                }
            ],
            "datacenter": {
                "name": "sao01",
            },
            "hourlyBillingFlag": True,
            "operatingSystemReferenceCode": "CENTOS_LATEST",
        },
        "virtualGuestMemberCount": 0,
        "status": {
            "id": 1,
            "keyName": "ACTIVE",
            "name": "Active"
        },
    },
    {
        "accountId": 31111,
        "cooldown": 1800,
        "createDate": "2018-04-24T04:22:00+08:00",
        "id": 224533333,
        "lastActionDate": "2019-01-19T04:53:18+08:00",
        "maximumMemberCount": 10,
        "minimumMemberCount": 0,
        "modifyDate": "2019-01-19T04:53:21+08:00",
        "name": "test-ajcb",
        "regionalGroupId": 1025,
        "virtualGuestMemberTemplate": {
            "accountId": 31111,
            "domain": "test.local",
            "hostname": "autoscale-ajcb01",
            "id": None,
            "maxCpu": None,
            "maxMemory": 1024,
            "postInstallScriptUri": "http://test.com",
            "startCpus": 1,
            "blockDevices": [
                {
                    "device": "0",
                    "diskImage": {
                        "capacity": 25,
                    }
                }
            ],
            "datacenter": {
                "name": "seo01",
            },
            "hourlyBillingFlag": True,
            "operatingSystemReferenceCode": "CENTOS_7_64",
        },
        "virtualGuestMemberCount": 0,
        "status": {
            "id": 1,
            "keyName": "ACTIVE",
            "name": "Active"
        }
    }
]

getPortableStorageVolumes = [
    {
        "capacity": 200,
        "createDate": "2018-10-06T04:27:59-06:00",
        "description": "Disk 2",
        "id": 11111,
        "modifyDate": "",
        "name": "Disk 2",
        "parentId": "",
        "storageRepositoryId": 22222,
        "typeId": 241,
        "units": "GB",
        "uuid": "fd477feb-bf32-408e-882f-02540gghgh111"
    }
]
getAllTopLevelBillingItems = [
    {
        "allowCancellationFlag": 1,
        "cancellationDate": "None",
        "categoryCode": "server",
        "createDate": "2015-05-28T09:53:41-06:00",
        "cycleStartDate": "2020-04-03T23:12:04-06:00",
        "description": "Dual E5-2690 v3 (12 Cores, 2.60 GHz)",
        "domainName": "sl-netbase.com",
        "hostName": "testsangeles101",
        "id": 53891943,
        "lastBillDate": "2020-04-03T23:12:04-06:00",
        "modifyDate": "2020-04-03T23:12:07-06:00",
        "nextBillDate": "2020-05-03T23:00:00-06:00",
        "orderItemId": 68626055,
        "parentId": "None",
        "recurringFee": "1000",
        "recurringFeeTaxRate": "0",
        "recurringMonths": 1,
        "hourlyFlag": False,
        "location": {
            "id": 265592,
            "longName": "Amsterdam 1",
            "name": "ams01",
            "statusId": 2
        },
        "nextInvoiceTotalRecurringAmount": 0,
        "orderItem": {
            "id": 68626055,
            "order": {
                "id": 4544893,
                "userRecord": {
                    "displayName": "TEst",
                    "email": "test@us.ibm.com",
                    "id": 167758,
                    "userStatus": {
                        "id": 1001,
                        "keyName": "CANCEL_PENDING",
                        "name": "Cancel Pending"
                    }
                }
            }
        },
        "resourceTableId": 544444
    },
    {
        "allowCancellationFlag": 1,
        "cancellationDate": "None",
        "categoryCode": "server",
        "createDate": "2015-05-28T09:56:44-06:00",
        "cycleStartDate": "2020-04-03T23:12:05-06:00",
        "description": "Dual E5-2690 v3 (12 Cores, 2.60 GHz)",
        "domainName": "sl-netbase.com",
        "hostName": "testsangeles101",
        "id": 53892197,
        "lastBillDate": "2020-04-03T23:12:05-06:00",
        "modifyDate": "2020-04-03T23:12:07-06:00",
        "nextBillDate": "2020-05-03T23:00:00-06:00",
        "orderItemId": 68626801,
        "recurringFee": "22220",
        "hourlyFlag": False,
        "location": {
            "id": 265592,
            "longName": "Amsterdam 1",
            "name": "ams01",
            "statusId": 2
        },
        "nextInvoiceTotalRecurringAmount": 0,
        "orderItem": {
            "id": 68626801,
            "order": {
                "id": 4545911,
                "userRecord": {
                    "displayName": "Test",
                    "email": "test@us.ibm.com",
                    "id": 167758,
                    "userStatus": {
                        "id": 1001,
                        "keyName": "ACTIVE",
                        "name": "Active"
                    }
                }
            }
        },
        "resourceTableId": 777777
    }
]

getNetworkStorage = [
    {
        "accountId": 1111111,
        "capacityGb": 20,
        "createDate": "2016-01-21T12:11:07-06:00",
        "id": 1234567,
        "nasType": "ISCSI",
        "username": "SL01SEL301234-11",
    },
    {
        "accountId": 1111111,
        "capacityGb": 20,
        "createDate": "2015-04-29T07:55:55-06:00",
        "id": 4917123,
        "nasType": "NAS",
        "username": "SL01SEV1234567_111"
    }
]

getRouters = [
    {
        "accountId": 1,
        "bareMetalInstanceFlag": 0,
        "domain": "softlayer.com",
        "fullyQualifiedDomainName": "fcr01a.ams01.softlayer.com",
        "hardwareStatusId": 5,
        "hostname": "fcr01a.ams01",
        "id": 123456,
        "serviceProviderId": 1,
        "topLevelLocation": {
            "id": 265592,
            "longName": "Amsterdam 1",
            "name": "ams01",
            "statusId": 2
        }
    }]

getNetworkStorage = [
    {
        "accountId": 1111111,
        "capacityGb": 20,
        "createDate": "2016-01-21T12:11:07-06:00",
        "id": 1234567,
        "nasType": "ISCSI",
        "username": "SL01SEL301234-11",
    },
    {
        "accountId": 1111111,
        "capacityGb": 20,
        "createDate": "2015-04-29T07:55:55-06:00",
        "id": 4917123,
        "nasType": "NAS",
        "username": "SL01SEV1234567_111"
    }
]

getNetworkMessageDeliveryAccounts = [
    {
        "accountId": 147258,
        "createDate": "2020-07-06T10:29:11-06:00",
        "id": 1232123,
        "typeId": 21,
        "username": "test_CLI@ie.ibm.com",
        "vendorId": 1,
        "type": {
            "description": "Delivery of messages through e-mail",
            "id": 21,
            "keyName": "EMAIL",
            "name": "Email"
        },
        "vendor": {
            "id": 1,
            "keyName": "SENDGRID",
            "name": "SendGrid"
        },
        "emailAddress": "test_CLI@ie.ibm.com",
        "smtpAccess": "1"
    }
]

getActiveAccountLicenses = [{
    "accountId": 123456,
    "capacity": "4",
    "key": "Y8GNS-7QRNG-OUIJO-MATEI-5GJRM",
    "units": "CPU",
    "billingItem": {
        "allowCancellationFlag": 1,
        "categoryCode": "software_license",
        "cycleStartDate": "2021-06-03T23:11:22-06:00",
        "description": "vCenter Server Appliance 6.0",
        "id": 741258963,
        "laborFee": "0",
        "laborFeeTaxRate": "0",
        "oneTimeFee": "0",
        "oneTimeFeeTaxRate": "0",
        "orderItemId": 963258741,
        "recurringFee": "0",
        "recurringFeeTaxRate": "0",
        "recurringMonths": 1,
        "serviceProviderId": 1,
        "setupFee": "0",
        "setupFeeTaxRate": "0"
    },
    "softwareDescription": {
        "controlPanel": 0,
        "id": 15963,
        "licenseTermValue": 0,
        "longDescription": "VMware vCenter 6.0",
        "manufacturer": "VMware",
        "name": "vCenter",
        "operatingSystem": 0,
        "version": "6.0",
        "virtualLicense": 0,
        "virtualizationPlatform": 0,
        "requiredUser": "administrator@vsphere.local"
    }
},
    {
        "accountId": 123456,
        "capacity": "4",
        "key": "TSZES-SJF85-04GLD-AXA64-8O1EO",
        "units": "CPU",
        "billingItem": {
            "allowCancellationFlag": 1,
            "categoryCode": "software_license",
            "description": "vCenter Server Appliance 6.x",
            "id": 36987456,
            "laborFee": "0",
            "laborFeeTaxRate": "0",
            "oneTimeFee": "0",
            "oneTimeFeeTaxRate": "0",
            "orderItemId": 25839,
            "recurringFee": "0",
            "recurringFeeTaxRate": "0",
            "recurringMonths": 1,
            "serviceProviderId": 1,
            "setupFee": "0",
            "setupFeeTaxRate": "0"
        },
        "softwareDescription": {
            "controlPanel": 0,
            "id": 1472,
            "licenseTermValue": 0,
            "longDescription": "VMware vCenter 6.0",
            "manufacturer": "VMware",
            "name": "vCenter",
            "operatingSystem": 0,
            "version": "6.0",
            "virtualLicense": 0,
            "virtualizationPlatform": 0,
            "requiredUser": "administrator@vsphere.local"
        }
    }
]

getActiveVirtualLicenses = [{
    "id": 12345,
    "ipAddress": "192.168.23.78",
    "key": "TEST.60220734.0000",
    "billingItem": {
        "categoryCode": "control_panel",
        "description": "Plesk Onyx (Linux) - (Unlimited) - VPS "
    },
    "softwareDescription": {
        "longDescription": "Plesk - Unlimited Domain w/ Power Pack for VPS 17.8.11 Linux",
        "manufacturer": "Plesk",
        "name": "Plesk - Unlimited Domain w/ Power Pack for VPS"
    },
    "subnet": {
        "broadcastAddress": "192.168.23.79",
        "cidr": 28,
        "gateway": "192.168.23.65",
        "id": 1973163,
        "isCustomerOwned": False,
        "isCustomerRoutable": False,
        "netmask": "255.255.255.240",
        "networkIdentifier": "128.116.23.64",
        "networkVlanId": 123456,
        "note": "test note",
        "sortOrder": "1",
        "subnetType": "ADDITIONAL_PRIMARY",
        "totalIpAddresses": "16",
        "usableIpAddressCount": "13",
        "version": 4
    }
}]

getBandwidthAllotments = [{
    'billingCyclePublicBandwidthUsage': {
        'amountIn': '6.94517',
        'amountOut': '6.8859'
    },
    "endDate": "2023-07-03T22:59:59-06:00",
    'id': 309961,
    'locationGroup': {
        'description': 'All Datacenters in Mexico',
        'id': 262,
        'locationGroupTypeId': 1,
        'name': 'MEX',
        'securityLevelId': None
    },
    'name': 'MexRegion',
    'projectedPublicBandwidthUsage': 9.88,
    'totalBandwidthAllocated': 3361
}]

getNetworkGateways = [{
    'id': 615448,
    'name': 'testFirewall-cgallo',
    'networkSpace': 'BOTH',
    'insideVlans': [],
    'members': [
        {
            'id': 687820,
            'hardware': {
                'hostname': 'dft03.pod03.dal13'
            }
        }
    ],
    'networkFirewall': {
        'id': 17438,
        'datacenter': {
            'name': 'dal13'
        },
        'firewallType': 'fortigate-security-appliance-10gb',
        'rules': []
    },
    'privateIpAddress': {
        'ipAddress': '10.37.115.70'
    },
    'publicIpAddress': {
        'ipAddress': '67.228.206.245'
    },
    'publicVlan': {
        'id': 3228726,
        'primaryRouter': {
            'hostname': 'fcr03a.dal13'
        }
    },
    'status': {
        'keyName': 'ACTIVE'
    }
}]

getPostProvisioningHooks = [{
    "id": 147258369,
    "name": "testProv",
    "uri": "http://test.com"
},
    {
        "id": 123654,
        "name": "GettestProv12345",
        "uri": "http://test.com"
    }]

getUpgradeRequests = [{
    "createDate": "2023-03-01T19:39:51-06:00",
    "id": 3237486,
    "maintenanceStartTimeUtc": "2023-03-01T19:39:51-06:00",
    "modifyDate": "2023-03-01T19:39:53-06:00",
    "orderId": 103127604,
    "statusId": 3,
    "ticketId": 152510472,
    "userId": 167758
},
    {
        "createDate": "2023-02-27T19:41:26-06:00",
        "id": 3229998,
        "maintenanceStartTimeUtc": "2023-02-27T19:41:26-06:00",
        "modifyDate": "2023-02-27T19:41:29-06:00",
        "orderId": 103029352,
        "statusId": 3,
        "ticketId": 152452042,
        "userId": 167758
    }]



"""Failback from a replicant volume."""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer
from SoftLayer.CLI import environment


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('volume-id')
@environment.pass_env
def cli(env, volume_id):
    """Failback a block volume from the given replica volume."""
    block_storage_manager = SoftLayer.BlockStorageManager(env.client)

    success = block_storage_manager.failback_from_replicant(volume_id)

    if success:
        click.echo("Failback from replicant is now in progress.")
    else:
        click.echo("Failback operation could not be initiated.")



"""Hardware internal Sensor ."""
# :license: MIT, see LICENSE for more details.

import SoftLayer
from SoftLayer.CLI import click
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--discrete', is_flag=True, default=False, help='Show discrete units associated hardware sensor')
@environment.pass_env
def cli(env, identifier, discrete):
    """Retrieve a server’s hardware state via its internal sensors."""
    mgr = SoftLayer.HardwareManager(env.client)
    sensors = mgr.get_sensors(identifier)

    temperature_table = formatting.Table(["Sensor", "Status", "Reading", "Critical Min", "Min", "Max", "Critical Max"],
                                         title='Temperature (c)')

    volts_table = formatting.Table(["Sensor", "Status", "Reading", "Critical Min", "Min", "Max", "Critical Max"],
                                   title='Volts')

    watts_table = formatting.Table(["Sensor", "Status", "Reading", "Critical Min", "Min", "Max", "Critical Max"],
                                   title='Watts')

    rpm_table = formatting.Table(["Sensor", "Status", "Reading", "Critical Min", "Min", "Max", "Critical Max"],
                                 title='RPM')

    discrete_table = formatting.Table(["Sensor", "Status", "Reading"],
                                      title='Discrete')

    for sensor in sensors:
        if sensor.get('sensorUnits') == 'degrees C':
            temperature_table.add_row([sensor.get('sensorId'),
                                       sensor.get('status'),
                                       sensor.get('sensorReading'),
                                       sensor.get('lowerCritical'),
                                       sensor.get('lowerNonCritical'),
                                       sensor.get('upperNonCritical'),
                                       sensor.get('upperCritical')])

        if sensor.get('sensorUnits') == 'Volts':
            volts_table.add_row([sensor.get('sensorId'),
                                 sensor.get('status'),
                                 sensor.get('sensorReading'),
                                 sensor.get('lowerCritical'),
                                 sensor.get('lowerNonCritical'),
                                 sensor.get('upperNonCritical'),
                                 sensor.get('upperCritical')])

        if sensor.get('sensorUnits') == 'Watts':
            watts_table.add_row([sensor.get('sensorId'),
                                 sensor.get('status'),
                                 sensor.get('sensorReading'),
                                 sensor.get('lowerCritical'),
                                 sensor.get('lowerNonCritical'),
                                 sensor.get('upperNonCritical'),
                                 sensor.get('upperCritical')])

        if sensor.get('sensorUnits') == 'RPM':
            rpm_table.add_row([sensor.get('sensorId'),
                               sensor.get('status'),
                               sensor.get('sensorReading'),
                               sensor.get('lowerCritical'),
                               sensor.get('lowerNonCritical'),
                               sensor.get('upperNonCritical'),
                               sensor.get('upperCritical')])

        if sensor.get('sensorUnits') == 'discrete':
            discrete_table.add_row([sensor.get('sensorId'),
                                    sensor.get('status'),
                                    sensor.get('sensorReading')])
    env.fout(temperature_table)
    env.fout(rpm_table)
    env.fout(volts_table)
    env.fout(watts_table)
    if discrete:
        env.fout(discrete_table)



createObject = {
    "createDate": "2022-10-18T09:27:48-06:00",
    "id": 78026761,
    "notes": "test slcli",
    "password": "123456",
    "softwareId": 67064532,
    "username": "testslcli",
    "software": {
        "hardwareId": 1532729,
        "id": 67064532,
        "manufacturerLicenseInstance": ""
    }
}



"""List scheduled snapshots of a specific volume"""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('volume_id')
@environment.pass_env
def cli(env, volume_id):
    """Lists snapshot schedules for a given volume."""

    block_manager = SoftLayer.BlockStorageManager(env.client)

    snapshot_schedules = block_manager.list_volume_schedules(volume_id)

    table = formatting.Table(['id',
                              'active',
                              'type',
                              'replication',
                              'date_created',
                              'minute',
                              'hour',
                              'day',
                              'week',
                              'day_of_week',
                              'date_of_month',
                              'month_of_year',
                              'maximum_snapshots'])

    for schedule in snapshot_schedules:

        if 'REPLICATION' in schedule['type']['keyname']:
            replication = '*'
        else:
            replication = formatting.blank()

        block_schedule_type = schedule['type']['keyname'].replace('REPLICATION_', '')
        block_schedule_type = block_schedule_type.replace('SNAPSHOT_', '')

        property_list = ['MINUTE', 'HOUR', 'DAY', 'WEEK',
                         'DAY_OF_WEEK', 'DAY_OF_MONTH',
                         'MONTH_OF_YEAR', 'SNAPSHOT_LIMIT']

        schedule_properties = []
        for prop_key in property_list:
            item = formatting.blank()
            for schedule_property in schedule.get('properties', []):
                if schedule_property['type']['keyname'] == prop_key:
                    if schedule_property['value'] == '-1':
                        item = '*'
                    else:
                        item = schedule_property['value']
                    break
            schedule_properties.append(item)

        table_row = [
            schedule['id'],
            '*' if schedule.get('active', '') else '',
            block_schedule_type,
            replication,
            schedule.get('createDate', '')]
        table_row.extend(schedule_properties)

        table.add_row(table_row)

    env.fout(table)



"""Get status for split or move completed percentage of a given file duplicate volume."""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand,
               epilog="""Get status for split or move completed percentage of a given file duplicate volume.""")
@click.argument('volume-id')
@environment.pass_env
def cli(env, volume_id):
    """Get status for split or move completed percentage of a given file duplicate volume."""
    table = formatting.Table(['Username', 'Active Conversion Start Timestamp', 'Completed Percentage'])

    file_manager = SoftLayer.FileStorageManager(env.client)

    value = file_manager.convert_dupe_status(volume_id)

    table.add_row(
        [
            value['volumeUsername'],
            value['activeConversionStartTime'],
            value['deDuplicateConversionPercentage']
        ]
    )

    env.fout(table)



"""
    SoftLayer.ssl
    ~~~~~~~~~~~~~
    SSL Manager/helpers

    :license: MIT, see LICENSE for more details.
"""


class SSLManager(object):
    """Manages SSL certificates in SoftLayer.

    See product information here: https://www.ibm.com/cloud/ssl-certificates

    Example::

       # Initialize the Manager.
       # env variables. These can also be specified in ~/.softlayer,
       # or passed directly to SoftLayer.Client()
       # SL_USERNAME = YOUR_USERNAME
       # SL_API_KEY = YOUR_API_KEY
       import SoftLayer
       client = SoftLayer.Client()
       mgr = SoftLayer.SSLManager(client)

    :param SoftLayer.API.BaseClient client: the client instance

    """

    def __init__(self, client):
        self.client = client
        self.ssl = self.client['Security_Certificate']

    def list_certs(self, method='all'):
        """List all certificates.

        :param string method: The type of certificates to list. Options are
                              'all', 'expired', and 'valid'.
        :returns: A list of dictionaries representing the requested SSL certs.

        Example::

            # Get all valid SSL certs
            certs = mgr.list_certs(method='valid')
            print certs

        """
        ssl = self.client['Account']
        methods = {
            'all': 'getSecurityCertificates',
            'expired': 'getExpiredSecurityCertificates',
            'valid': 'getValidSecurityCertificates'
        }

        mask = "mask[id, commonName, validityDays, notes]"
        func = getattr(ssl, methods[method])
        return func(mask=mask)

    def add_certificate(self, certificate):
        """Creates a new certificate.

        :param dict certificate: A dictionary representing the parts of the
                                 certificate.
                                 See sldn.softlayer.com for more info.

        Example::

            cert = ??
            result = mgr.add_certificate(certificate=cert)

        """
        return self.ssl.createObject(certificate)

    def remove_certificate(self, cert_id):
        """Removes a certificate.

        :param integer cert_id: a certificate ID to remove

        Example::

            # Removes certificate with id 1234
            result = mgr.remove_certificate(cert_id = 1234)
        """
        return self.ssl.deleteObject(id=cert_id)

    def edit_certificate(self, certificate):
        """Updates a certificate with the included options.

        The provided dict must include an 'id' key and value corresponding to
        the certificate ID that should be updated.

        :param dict certificate: the certificate to update.

        Example::

            # Updates the cert id 1234
            cert['id'] = 1234
            cert['certificate'] = ??
            result = mgr.edit_certificate(certificate=cert)

        """
        return self.ssl.editObject(certificate, id=certificate['id'])

    def get_certificate(self, cert_id):
        """Gets a certificate with the ID specified.

        :param integer cert_id: the certificate ID to retrieve

        Example::

            cert = mgr.get_certificate(cert_id=1234)
            print(cert)

        """
        return self.ssl.getObject(id=cert_id)



"""Get Event Logs."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer import exceptions
from SoftLayer import utils


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.option('--date-min', '-d',
              help='The earliest date we want to search for event logs in mm/dd/yyyy format.')
@click.option('--date-max', '-D',
              help='The latest date we want to search for event logs in mm/dd/yyyy format.')
@click.option('--obj-event', '-e',
              help="The event we want to get event logs for")
@click.option('--obj-id', '-i',
              help="The id of the object we want to get event logs for")
@click.option('--obj-type', '-t',
              help="The type of the object we want to get event logs for")
@click.option('--utc-offset', '-z', default='-0000', show_default=True,
              help="UTC Offset for searching with dates. +/-HHMM format")
@click.option('--metadata/--no-metadata', default=False, show_default=True,
              help="Display metadata if present")
@click.option('--limit', '-l', type=click.INT, default=50, show_default=True,
              help="Total number of result to return. -1 to return ALL, there may be a LOT of these.")
@environment.pass_env
def cli(env, date_min, date_max, obj_event, obj_id, obj_type, utc_offset, metadata, limit):
    """Get Event Logs

    Example::

        slcli event-log get -d 01/01/2019 -D 02/01/2019 -t User -l 10
    """
    columns = ['Event', 'Object', 'Type', 'Date', 'Username']

    event_mgr = SoftLayer.EventLogManager(env.client)
    user_mgr = SoftLayer.UserManager(env.client)
    request_filter = event_mgr.build_filter(date_min, date_max, obj_event, obj_id, obj_type, utc_offset)
    logs = event_mgr.get_event_logs(request_filter)
    log_time = "%Y-%m-%dT%H:%M:%S.%f%z"
    # A list of IDs that we have looked up user information about.
    user_data = {}

    if metadata:
        columns.append('Metadata')

    row_count = 0
    click.secho(", ".join(columns))
    for log in logs:
        if isinstance(log, str) or log is None:
            click.secho('No logs available for filter %s.' % request_filter, fg='red')
            return

        user = log['userType']
        label = log.get('label', '')
        if user == "CUSTOMER":
            username = user_data.get(log['userId'])
            if username is None:
                try:
                    username = user_mgr.get_user(log['userId'], "mask[username]")['username']
                # Some users might not be able to access information about this specific userId
                except exceptions.SoftLayerAPIError:
                    username = log['userId']
            # This keeps track of any users we asked the API about, so we can make fewer calls.
            user_data[log['userId']] = username
            user = username

        if metadata:
            metadata_data = log['metaData'].strip("\n\t")

            click.secho(f"'{log['eventName']}','{label}','{log['objectName']}',"
                        f"'{utils.clean_time(log['eventCreateDate'], in_format=log_time)}',"
                        f"'{user}','{metadata_data}'")
        else:
            click.secho(f"'{log['eventName']}','{label}','{log['objectName']}',"
                        f"'{utils.clean_time(log['eventCreateDate'], in_format=log_time)}',"
                        f"'{user}'")

        row_count = row_count + 1
        if row_count >= limit and limit != -1:
            return



"""Disable/Enable snapshots space usage threshold warning for a specific volume"""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer
from SoftLayer.CLI import environment


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('volume_id')
@click.option(
    '--enable/--disable',
    default=True,
    help='Enable/Disable snapshot notification. Use `slcli file snapshot-set-notification volumeId --enable` to enable',
    required=True)
@environment.pass_env
def cli(env, volume_id, enable):
    """Enables/Disables snapshot space usage threshold warning for a given volume"""
    file_manager = SoftLayer.FileStorageManager(env.client)

    status = file_manager.set_volume_snapshot_notification(volume_id, enable)
    if status:
        click.echo(
            'Snapshots space usage threshold warning notification has bee set to %s for volume %s'
            % (enable, volume_id))



updateLoadBalancerHealthMonitors = {
    'backendPort': 80,
    'backendProtocol': 'HTTP',
    'healthMonitorUuid': '1a1aa111-4474-4e16-9f02-4de959244444',
    'interval': 50,
    'maxRetries': 10,
    'timeout': 10,
    'urlPath': None
}



"""Grants a user access to a given device"""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions


@click.command(cls=SLCommand, )
@click.argument('identifier')
@click.option('--hardware', help="Hardware ID")
@click.option('--virtual', help="Virtual Guest ID")
@click.option('--dedicated', help="dedicated host ID")
@environment.pass_env
def cli(env, identifier, hardware, virtual, dedicated):
    """Grants a user access to a given device.

    Example: slcli user grant-access 123456 --hardware 123456789
    """

    mgr = SoftLayer.UserManager(env.client)
    result = False
    if hardware:
        result = mgr.grant_hardware_access(identifier, hardware)
        if result:
            click.secho(f"User {identifier} has been given access to hardware {hardware}", fg='green')

    if virtual:
        result = mgr.grant_virtual_access(identifier, virtual)
        if result:
            click.secho(f"User {identifier} has been given access to hardware {virtual}", fg='green')

    if dedicated:
        result = mgr.grant_dedicated_access(identifier, dedicated)
        if result:
            click.secho(f"User {identifier} has been given access to hardware {dedicated}", fg='green')

    if not result:
        raise exceptions.CLIAbort('A device option is required.\n'
                                  'E.g slcli user grant-access 123456 --hardware 91803794\n'
                                  '    slcli user grant-access 123456 --dedicated 91803793\n'
                                  '    slcli user grant-access 123456 --virtual 91803792')



"""Cancel a dedicated server."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--immediate',
              is_flag=True,
              default=False,
              help="Cancels the server immediately (instead of on the billing anniversary)")
@click.option('--comment',
              help="An optional comment to add to the cancellation ticket")
@click.option('--reason',
              help="An optional cancellation reason. See cancel-reasons for a list of available options")
@click.option('--force',  default=False, is_flag=True, help="Force modify")
@environment.pass_env
def cli(env, identifier, immediate, comment, reason, force):
    """Cancel a dedicated server."""

    mgr = SoftLayer.HardwareManager(env.client)
    hw_id = helpers.resolve_id(mgr.resolve_ids, identifier, 'hardware')

    if not (env.skip_confirmations or formatting.no_going_back(hw_id)):
        raise exceptions.CLIAbort('Aborted')

    if not force:
        if not (env.skip_confirmations or
                formatting.confirm("This action will incur charges on your account. Continue?")):
            raise exceptions.CLIAbort('Aborted')

    mgr.cancel_hardware(hw_id, reason, comment, immediate)



"""Lists account orders."""
# :license: MIT, see LICENSE for more details.

import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.managers.account import AccountManager as AccountManager
from SoftLayer import utils


def upgrade_table(upgrades):
    """Formats a table for upgrade orders"""
    table = formatting.Table(['Id', 'Maintance window', 'Status', 'Created Date',
                              'Case'], title="Upgrade orders")
    table.align['Subject'] = 'l'
    table.align['Impacted Resources'] = 'l'
    for upgrade in upgrades:
        table.add_row([upgrade.get('id'),
                       upgrade.get('maintenanceStartTimeUtc'),
                       upgrade.get('statusId'),
                       upgrade.get('createDate'),
                       upgrade.get('ticketId') or '--'])
    return table


@click.command(cls=SLCommand)
@click.option('--limit', '-l',
              help='How many results to get in one api call',
              default=100,
              show_default=True)
@click.option('--upgrades', is_flag=True, default=False,
              help="Show upgrades orders.")
@environment.pass_env
def cli(env, limit, upgrades):
    """Lists account orders. Use `slcli order lookup <ID>` to find more details about a specific order."""
    manager = AccountManager(env.client)
    orders = manager.get_account_all_billing_orders(limit)
    upgrade = manager.get_account_upgrade_orders(limit)

    order_table = formatting.Table(['Id', 'State', 'User', 'Date', 'Amount', 'Item'],
                                   title="orders")
    order_table.align = 'l'

    for order in orders:
        items = []
        for item in order['items']:
            items.append(item['description'])
        create_date = utils.clean_time(order['createDate'], in_format='%Y-%m-%d', out_format='%Y-%m-%d')

        order_table.add_row([order['id'], order['status'], order['userRecord']['username'], create_date,
                             order['orderTotalAmount'], utils.trim_to(' '.join(map(str, items)), 50)])
    env.fout(order_table)
    if upgrades:
        env.fout(upgrade_table(upgrade))



"""Security group interface operations."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting

COLUMNS = ['networkComponentId',
           'virtualServerId',
           'hostname',
           'interface',
           'ipAddress', ]

REQUEST_COLUMNS = ['requestId']


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('securitygroup_id')
@click.option('--sortby',
              help='Column to sort by',
              type=click.Choice(COLUMNS))
@environment.pass_env
def interface_list(env, securitygroup_id, sortby):
    """List interfaces associated with security groups."""
    mgr = SoftLayer.NetworkManager(env.client)

    table = formatting.Table(COLUMNS)
    table.sortby = sortby

    mask = (
        '''networkComponentBindings[
            networkComponentId,
            networkComponent[
                id,
                port,
                guest[
                    id,
                    hostname,
                    primaryBackendIpAddress,
                    primaryIpAddress
                ]
            ]
        ]'''
    )

    secgroup = mgr.get_securitygroup(securitygroup_id, mask=mask)
    for binding in secgroup.get('networkComponentBindings', []):
        interface_id = binding['networkComponentId']
        try:
            interface = binding['networkComponent']
            vsi = interface['guest']
            vsi_id = vsi['id']
            hostname = vsi['hostname']
            priv_pub = 'PRIVATE' if interface['port'] == 0 else 'PUBLIC'
            ip_address = (vsi['primaryBackendIpAddress']
                          if interface['port'] == 0
                          else vsi['primaryIpAddress'])
        except KeyError:
            vsi_id = "N/A"
            hostname = "Not enough permission to view"
            priv_pub = "N/A"
            ip_address = "N/A"

        table.add_row([
            interface_id,
            vsi_id,
            hostname,
            priv_pub,
            ip_address
        ])

    env.fout(table)


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('securitygroup_id')
@click.option('--network-component', '-n',
              help=('The network component to associate '
                    'with the security group'))
@click.option('--server', '-s',
              help='The server ID to associate with the security group')
@click.option('--interface', '-i',
              help='The interface of the server to associate (public/private)')
@environment.pass_env
def add(env, securitygroup_id, network_component, server, interface):
    """Attach an interface to a security group."""
    _validate_args(network_component, server, interface)

    mgr = SoftLayer.NetworkManager(env.client)
    component_id = _get_component_id(env, network_component, server, interface)

    ret = mgr.attach_securitygroup_component(securitygroup_id,
                                             component_id)
    if not ret:
        raise exceptions.CLIAbort("Could not attach network component")

    table = formatting.Table(REQUEST_COLUMNS)
    table.add_row([ret['requestId']])

    env.fout(table)


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('securitygroup_id')
@click.option('--network-component', '-n',
              help=('The network component to remove from '
                    'with the security group'))
@click.option('--server', '-s',
              help='The server ID to remove from the security group')
@click.option('--interface', '-i',
              help='The interface of the server to remove (public/private)')
@environment.pass_env
def remove(env, securitygroup_id, network_component, server, interface):
    """Detach an interface from a security group."""
    _validate_args(network_component, server, interface)

    mgr = SoftLayer.NetworkManager(env.client)
    component_id = _get_component_id(env, network_component, server, interface)

    ret = mgr.detach_securitygroup_component(securitygroup_id,
                                             component_id)
    if not ret:
        raise exceptions.CLIAbort("Could not detach network component")

    table = formatting.Table(REQUEST_COLUMNS)
    table.add_row([ret['requestId']])

    env.fout(table)


def _validate_args(network_component, server, interface):
    use_server = bool(server and interface and not network_component)
    use_component = bool(network_component and not bool(server or interface))

    if not use_server and not use_component:
        raise exceptions.CLIAbort("Must set either --network-component "
                                  "or both --server and --interface")
    if use_server and interface.lower() not in ['public', 'private']:
        raise exceptions.CLIAbort(
            "Interface must be either 'public' or 'private'")


def _get_component_id(env, network_component, server, interface):
    use_server = bool(server and interface and not network_component)
    vs_mgr = SoftLayer.VSManager(env.client)

    if use_server:
        vs_mask = 'networkComponents[id, port]'
        vsi = vs_mgr.get_instance(server, mask=vs_mask)
        port = 0 if interface.lower() == 'private' else 1
        component = [c for c in vsi['networkComponents'] if c['port'] == port]
        if len(component) != 1:
            raise exceptions.CLIAbort("Instance %s has no %s interface"
                                      % (server, interface))
        component_id = component[0]['id']
    else:
        component_id = network_component

    return component_id



"""List package items."""
# :license: MIT, see LICENSE for more details.
import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.managers import ordering
from SoftLayer.utils import lookup

COLUMNS = ['Category', 'KeyName', 'Description', 'Price Id']
COLUMNS_ITEM_PRICES = ['KeyName', 'Price Id', 'Hourly', 'Monthly', 'Restriction']
COLUMNS_ITEM_PRICES_LOCATION = ['KeyName', 'Price Id', 'Hourly', 'Monthly', 'Restriction']


@click.command(cls=SLCommand)
@click.argument('package_keyname')
@click.option('--keyword', '-k', help="A word (or string) used to filter item names.")
@click.option('--category', '-c', help="Category code to filter items by")
@click.option('--prices', '-p', is_flag=True,
              help='Use --prices to list the server item prices, and to list the '
                   'Item Prices by location, add it to the --prices option using '
                   'location KeyName, e.g. --prices AMSTERDAM02')
@click.argument('location', required=False)
@environment.pass_env
def cli(env, package_keyname, keyword, category, prices, location=None):
    """List package items used for ordering.

    The item keyNames listed can be used with `slcli order place` to specify
    the items that are being ordered in the package.

    .. Note::
        Items with a numbered category, like disk0 or gpu0, can be included
        multiple times in an order to match how many of the item you want to order.

    ::

        # List all items in the VSI package
        slcli order item-list CLOUD_SERVER

        # List Ubuntu OSes from the os category of the Bare Metal package
        slcli order item-list BARE_METAL_SERVER --category os --keyword ubuntu

    """
    manager = ordering.OrderingManager(env.client)

    tables = []

    _filter = {'items': {}}
    if keyword:
        _filter['items']['description'] = {'operation': '*= %s' % keyword}
    if category:
        _filter['items']['categories'] = {'categoryCode': {'operation': '_= %s' % category}}

    items = manager.list_items(package_keyname, filter=_filter)
    sorted_items = sort_items(items)

    categories = sorted_items.keys()
    if prices:
        _item_list_prices(categories, sorted_items, tables)
        if location:
            location_prices = manager.get_item_prices_by_location(location, package_keyname)
            _location_item_prices(location_prices, location, tables)
    else:
        table_items_detail = formatting.Table(COLUMNS)
        for category_name in sorted(categories):
            for item in sorted_items[category_name]:
                table_items_detail.add_row([category_name, item['keyName'], item['description'], get_price(item)])
        tables.append(table_items_detail)
    env.fout(tables)


def sort_items(items):
    """sorts the items into a dictionary of categories, with a list of items"""

    sorted_items = {}
    for item in items:
        category = lookup(item, 'itemCategory', 'categoryCode')
        if sorted_items.get(category) is None:
            sorted_items[category] = []
        sorted_items[category].append(item)

    return sorted_items


def get_price(item):
    """Given an SoftLayer_Product_Item, returns its default price id"""

    for price in item.get('prices', []):
        if not price.get('locationGroupId'):
            return price.get('id')
    return 0


def _item_list_prices(categories, sorted_items, tables):
    """Add the item prices cost and capacity restriction to the table"""
    table_prices = formatting.Table(COLUMNS_ITEM_PRICES)
    for category in sorted(categories):
        for item in sorted_items[category]:
            for price in item['prices']:
                if not price.get('locationGroupId'):
                    cr_max = get_item_price_data(price, 'capacityRestrictionMaximum')
                    cr_min = get_item_price_data(price, 'capacityRestrictionMinimum')
                    cr_type = get_item_price_data(price, 'capacityRestrictionType')
                    table_prices.add_row([item['keyName'], price['id'],
                                          get_item_price_data(price, 'hourlyRecurringFee'),
                                          get_item_price_data(price, 'recurringFee'),
                                          "%s - %s %s" % (cr_min, cr_max, cr_type)])
    tables.append(table_prices)


def get_item_price_data(price, item_attribute):
    """Given an SoftLayer_Product_Item_Price, returns its default price data"""
    result = '-'
    if item_attribute in price:
        result = price[item_attribute]
    return result


def _location_item_prices(location_prices, location, tables):
    """Add a location prices table to tables.

    :param list location_prices : Location prices.
    :param string location : Location.
    :param list tables: Table list to add location prices table.
    """
    location_prices_table = formatting.Table(COLUMNS_ITEM_PRICES_LOCATION, title="Item Prices for %s" % location)
    location_prices_table.sortby = 'KeyName'
    location_prices_table.align = 'l'
    for price in location_prices:
        cr_max = get_item_price_data(price, 'capacityRestrictionMaximum')
        cr_min = get_item_price_data(price, 'capacityRestrictionMinimum')
        cr_type = get_item_price_data(price, 'capacityRestrictionType')
        location_prices_table.add_row(
            [price['item']['keyName'], price['id'],
             get_item_price_data(price, 'hourlyRecurringFee'),
             get_item_price_data(price, 'recurringFee'),
             "%s - %s %s" % (cr_min, cr_max, cr_type)])
    tables.append(location_prices_table)



getAllObjects = [{'id': 21,
                  'keyname': 'CONSISTENT_HASH_IP',
                  'name': 'Consistent Hash IP'},
                 {'id': 9,
                  'keyname': 'INSERT_COOKIE',
                  'name': 'Insert Cookie'},
                 {'id': 1,
                  'keyname': 'LEAST_CONNECTIONS',
                  'name': 'Least Connections'},
                 {'id': 7,
                  'keyname': 'LEAST_CONNECTIONS_INSERT_COOKIE',
                  'name': 'Least Connections with Inserted Cookie'},
                 {'id': 4,
                  'keyname': 'LEAST_CONNECTIONS_PERSISTENT_IP',
                  'name': 'Least Connections with Persistent IP'},
                 {'id': 11,
                  'keyname': 'PERSISTENT_IP',
                  'name': 'Persistent IP'},
                 {'id': 10,
                  'keyname': 'ROUND_ROBIN',
                  'name': 'Round Robin'},
                 {'id': 6,
                  'keyname': 'ROUND_ROBIN_INSERT_COOKIE',
                  'name': 'Round Robin with Inserted Cookie'},
                 {'id': 3,
                  'keyname': 'ROUND_ROBIN_PERSISTENT_IP',
                  'name': 'Round Robin with Persistent IP'},
                 {'id': 2,
                  'keyname': 'SHORTEST_RESPONSE',
                  'name': 'Shortest Response'},
                 {'id': 8,
                  'keyname': 'SHORTEST_RESPONSE_INSERT_COOKIE',
                  'name': 'Shortest Response with Inserted Cookie'},
                 {'id': 5,
                  'keyname': 'SHORTEST_RESPONSE_PERSISTENT_IP',
                  'name': 'Shortest Response with Persistent IP'}]



"""View details of a placement group"""

import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers
from SoftLayer.managers.vs_placement import PlacementManager as PlacementManager


@click.command(cls=SLCommand, epilog="Once provisioned, virtual guests can be managed with the slcli vs commands")
@click.argument('identifier')
@environment.pass_env
def cli(env, identifier):
    """View details of a placement group.

    IDENTIFIER can be either the Name or Id of the placement group you want to view
    """
    manager = PlacementManager(env.client)
    group_id = helpers.resolve_id(manager.resolve_ids, identifier, 'placement_group')
    result = manager.get_object(group_id)
    table = formatting.Table(["Id", "Name", "Backend Router", "Rule", "Created"])

    table.add_row([
        result['id'],
        result['name'],
        result['backendRouter']['hostname'],
        result['rule']['name'],
        result['createDate']
    ])
    guest_table = formatting.Table([
        "Id",
        "FQDN",
        "Primary IP",
        "Backend IP",
        "CPU",
        "Memory",
        "Provisioned",
        "Transaction"
    ])
    for guest in result['guests']:
        guest_table.add_row([
            guest.get('id'),
            guest.get('fullyQualifiedDomainName'),
            guest.get('primaryIpAddress'),
            guest.get('primaryBackendIpAddress'),
            guest.get('maxCpu'),
            guest.get('maxMemory'),
            guest.get('provisionDate'),
            formatting.active_txn(guest)
        ])

    env.fout(table)
    env.fout(guest_table)



"""Edit bandwidth pool."""
# :license: MIT, see LICENSE for more details.
import click

from SoftLayer import BandwidthManager
from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer import utils

location_groups = {
    "SJC/DAL/WDC/TOR/MON": "US/Canada",
    "AMS/LON/MAD/PAR": "AMS/LON/MAD/PAR",
    "SNG/HKG/OSA/TOK": "SNG/HKG/JPN",
    "SYD": "AUS",
    "MEX": "MEX",
    "SAO": "BRA",
    "CHE": "IND",
    "MIL": "ITA",
    "SEO": "KOR",
    "FRA": "FRA"
}


@click.command(cls=SLCommand)
@click.argument('identifier')
@click.option('--name', required=True, help="Pool name")
@environment.pass_env
def cli(env, identifier, name):
    """Edit bandwidth pool."""

    manager = BandwidthManager(env.client)
    bandwidth_pool = manager.edit_pool(identifier, name)

    if bandwidth_pool:

        edited_pool = manager.get_bandwidth_detail(identifier)
        locations = manager.get_location_group()

        location = next(
            (location for location in locations if location['id'] == edited_pool.get('locationGroupId')), None)

        region_name = next((key for key, value in location_groups.items() if value == location.get('name')), None)

        table = formatting.KeyValueTable(['Name', 'Value'])
        table.add_row(['Id', edited_pool.get('id')])
        table.add_row(['Name Pool', name])
        table.add_row(['Region', region_name])
        table.add_row(['Created Date', utils.clean_time(edited_pool.get('createDate'))])
        env.fout(table)



"""List server credentials."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers
from SoftLayer import exceptions


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@environment.pass_env
def cli(env, identifier):
    """List server credentials."""

    manager = SoftLayer.HardwareManager(env.client)
    hardware_id = helpers.resolve_id(manager.resolve_ids,
                                     identifier,
                                     'hardware')
    instance = manager.get_hardware(hardware_id)

    table = formatting.Table(['Username', 'Password', 'Software', 'Version'])
    for item in instance['softwareComponents']:
        if 'passwords' not in item:
            raise exceptions.SoftLayerError("No passwords found in softwareComponents")
        for credentials in item['passwords']:
            table.add_row([credentials.get('username', 'None'),
                           credentials.get('password', 'None'),
                           item['softwareLicense']['softwareDescription']['name'],
                           item['softwareLicense']['softwareDescription']['version']])
    env.fout(table)



"""
    SoftLayer.CLI.routes
    ~~~~~~~~~~~~~~~~~~~
    Routes for shell-specific commands

    :license: MIT, see LICENSE for more details.
"""

ALL_ROUTES = [
    ('exit', 'SoftLayer.shell.cmd_exit:cli'),
    ('shell-help', 'SoftLayer.shell.cmd_help:cli'),
    ('env', 'SoftLayer.shell.cmd_env:cli'),
]

ALL_ALIASES = {
    '?': 'shell-help',
    'help': 'shell-help',
    'quit': 'exit',
}



getMappingUsageMetrics = [
    {
        "names": [
            "TotalBandwidth",
            "TotalHits",
            "HitRatio"
        ],
        "totals": [
            1.0,
            3,
            2.0
        ],
        "type": "TOTALS"
    }
]



"""Command lines which modify power states."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@environment.pass_env
def rescue(env, identifier):
    """Reboot into a rescue image."""

    vsi = SoftLayer.VSManager(env.client)
    vs_id = helpers.resolve_id(vsi.resolve_ids, identifier, 'VS')
    if not (env.skip_confirmations or
            formatting.confirm("This action will reboot this VSI. Continue?")):
        raise exceptions.CLIAbort('Aborted')

    vsi.rescue(vs_id)


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--hard/--soft',
              default=None,
              help="Perform a hard or soft reboot")
@environment.pass_env
def reboot(env, identifier, hard):
    """Reboot an active virtual server."""

    virtual_guest = env.client['Virtual_Guest']
    mgr = SoftLayer.VSManager(env.client)
    vs_id = helpers.resolve_id(mgr.resolve_ids, identifier, 'VS')
    if not (env.skip_confirmations or
            formatting.confirm(f'This will reboot the VS with id {vs_id}. '
                               'Continue?')):
        raise exceptions.CLIAbort('Aborted.')

    if hard is True:
        virtual_guest.rebootHard(id=vs_id)
    elif hard is False:
        virtual_guest.rebootSoft(id=vs_id)
    else:
        virtual_guest.rebootDefault(id=vs_id)


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--hard/--soft', help="Perform a hard shutdown")
@environment.pass_env
def power_off(env, identifier, hard):
    """Power off an active virtual server."""

    virtual_guest = env.client['Virtual_Guest']
    vsi = SoftLayer.VSManager(env.client)
    vs_id = helpers.resolve_id(vsi.resolve_ids, identifier, 'VS')
    if not (env.skip_confirmations or
            formatting.confirm(f'This will power off the VS with id {vs_id}. '
                               'Continue?')):
        raise exceptions.CLIAbort('Aborted.')

    if hard:
        virtual_guest.powerOff(id=vs_id)
    else:
        virtual_guest.powerOffSoft(id=vs_id)


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@environment.pass_env
def power_on(env, identifier):
    """Power on a virtual server."""

    vsi = SoftLayer.VSManager(env.client)
    vs_id = helpers.resolve_id(vsi.resolve_ids, identifier, 'VS')
    env.client['Virtual_Guest'].powerOn(id=vs_id)


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@environment.pass_env
def pause(env, identifier):
    """Pauses an active virtual server."""

    vsi = SoftLayer.VSManager(env.client)
    vs_id = helpers.resolve_id(vsi.resolve_ids, identifier, 'VS')

    if not (env.skip_confirmations or
            formatting.confirm(f'This will pause the VS with id {vs_id}. Continue?')):
        raise exceptions.CLIAbort('Aborted.')

    env.client['Virtual_Guest'].pause(id=vs_id)


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@environment.pass_env
def resume(env, identifier):
    """Resumes a paused virtual server."""

    vsi = SoftLayer.VSManager(env.client)
    vs_id = helpers.resolve_id(vsi.resolve_ids, identifier, 'VS')
    env.client['Virtual_Guest'].resume(id=vs_id)



"""Find details about this machine."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting

META_CHOICES = [
    'backend_ip',
    'backend_mac',
    'datacenter',
    'datacenter_id',
    'fqdn',
    'frontend_mac',
    'id',
    'ip',
    'network',
    'provision_state',
    'tags',
    'user_data',
]

META_MAPPING = {
    'backend_ip': 'primary_backend_ip',
    'ip': 'primary_ip',
}

NL = "\n    "
HELP = f"""Find details about the machine making these API calls.

.. csv-table:: Choices

    {NL.join(META_CHOICES)}

"""


@click.command(cls=SoftLayer.CLI.command.SLCommand, help=HELP,
               short_help="Find details about this machine.",
               epilog="These commands only work on devices on the backend "
                      "SoftLayer network. This allows for self-discovery for "
                      "newly provisioned resources.")
@click.argument('prop', type=click.Choice(META_CHOICES))
@environment.pass_env
def cli(env, prop):
    """Find details about this machine."""

    try:
        if prop == 'network':
            env.fout(get_network())
            return

        meta_prop = META_MAPPING.get(prop) or prop
        env.fout(SoftLayer.MetadataManager().get(meta_prop))
    except SoftLayer.TransportError as ex:
        message = 'Cannot connect to the backend service address. Make sure '\
                  'this command is being ran from a device on the backend network.'
        raise exceptions.CLIAbort(message) from ex


def get_network():
    """Returns a list of tables with public and private network details."""
    meta = SoftLayer.MetadataManager()
    network_tables = []
    for network_func in [meta.public_network, meta.private_network]:
        network = network_func()

        table = formatting.KeyValueTable(['name', 'value'])
        table.align['name'] = 'r'
        table.align['value'] = 'l'
        table.add_row(['mac addresses',
                       formatting.listing(network['mac_addresses'],
                                          separator=',')])
        table.add_row(['router', network['router']])
        table.add_row(['vlans',
                       formatting.listing(network['vlans'], separator=',')])
        table.add_row(['vlan ids',
                       formatting.listing(network['vlan_ids'], separator=',')])
        network_tables.append(table)

    return network_tables



createL7Pool = {'accountId': 1234,
                'address': '01-307608-ams01.clb.appdomain.cloud',
                'createDate': '2019-08-12T07:49:43-06:00',
                'id': 1111111,
                'isPublic': 0,
                'locationId': 265592,
                'modifyDate': '2019-08-13T16:26:06-06:00',
                'name': 'dcabero-01',
                'operatingStatus': 'ONLINE',
                'provisioningStatus': 'ACTIVE',
                'type': 0,
                'useSystemPublicIpPool': 1,
                'uuid': '1a1aa111-4474-4e16-9f02-4de959229b85',
                'listenerCount': 4,
                'memberCount': 1,
                'datacenter': {
                    'id': 265592,
                    'longName': 'Amsterdam 1',
                    'name': 'ams01',
                    'statusId': 2
                }}
deleteObject = {'accountId': 1234,
                'address': '01-307608-ams01.clb.appdomain.cloud',
                'createDate': '2019-08-12T07:49:43-06:00',
                'id': 1111111,
                'isPublic': 0,
                'locationId': 265592,
                'modifyDate': '2019-08-13T16:26:06-06:00',
                'name': 'dcabero-01',
                'operatingStatus': 'ONLINE',
                'provisioningStatus': 'ACTIVE',
                'type': 0,
                'useSystemPublicIpPool': 1,
                'uuid': '1a1aa111-4474-4e16-9f02-4de959229b85',
                'listenerCount': 4,
                'memberCount': 1,
                'datacenter': {
                    'id': 265592,
                    'longName': 'Amsterdam 1',
                    'name': 'ams01',
                    'statusId': 2
                }}



cancelService = True
cancelServiceOnAnniversaryDate = True
cancelItem = True

getObject = {
    "allowCancellationFlag": 1,
    "cancellationDate": "None",
    "categoryCode": "server",
    "createDate": "2015-05-28T10:36:38-06:00",
    "cycleStartDate": "2020-04-03T23:12:05-06:00",
    "description": "Dual E5-2690 v3 (12 Cores, 2.60 GHz)",
    "domainName": "sl-test.com",
    "hostName": "testsangeles101",
    "id": 53897671,
    "lastBillDate": "2020-04-03T23:12:05-06:00",
    "modifyDate": "2020-04-03T23:12:07-06:00",
    "nextBillDate": "2020-05-03T23:00:00-06:00",
    "orderItemId": 68634907,
    "parentId": "None",
    "recurringFee": "1000",
    "recurringMonths": 1,
    "children": [
        {
            "allowCancellationFlag": 1,
            "associatedBillingItemId": "53897671",
            "cancellationDate": "None",
            "categoryCode": "second_processor",
            "createDate": "2015-05-28T10:36:38-06:00",
            "cycleStartDate": "2020-04-03T23:12:05-06:00",
            "description": "E5-2690 v3 (12 Cores, 2.60 GHz)",
            "id": 53897673,
            "lastBillDate": "2020-04-03T23:12:05-06:00",
            "modifyDate": "2020-04-03T23:12:07-06:00",
            "nextBillDate": "2020-05-03T23:00:00-06:00",
            "oneTimeFee": "0",
            "oneTimeFeeTaxRate": "0",
            "orderItemId": 68634909,
            "parentId": 53897671,
            "recurringFee": "1000",
            "setupFeeTaxRate": "0"
        },
    ],
    "hourlyFlag": False,
    "location": {
        "id": 265592,
        "longName": "Amsterdam 1",
        "name": "ams01",
        "statusId": 2
    },
    "nextInvoiceTotalRecurringAmount": 0,
    "orderItem": {
        "id": 68634907,
        "order": {
            "id": 4546175,
            "userRecord": {
                "displayName": "Tester",
                "email": "test@us.ibm.com",
                "id": 167758,
                "userStatus": {
                    "keyName": "ACTIVE",
                    "name": "Active"
                }
            }
        }
    },
    "resourceTableId": "None"
}



"""Enable or Disable specific noticication for the current user"""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.option('--enable/--disable', default=True,
              help="Enable (DEFAULT) or Disable selected notification")
@click.argument('notification', nargs=-1, required=True)
@environment.pass_env
def cli(env, enable, notification):
    """Enable or Disable specific notifications for the active user.

    Notification names should be enclosed in quotation marks.
    Example::

        slcli user edit-notifications --enable 'Order Approved' 'Reload Complete'

    """

    mgr = SoftLayer.UserManager(env.client)

    if enable:
        result = mgr.enable_notifications(notification)
    else:
        result = mgr.disable_notifications(notification)

    if result:
        click.secho(f"Notifications updated successfully: {', '.join(notification)}", fg='green')
    else:
        click.secho(f"Failed to update notifications: {', '.join(notification)}", fg='red')



"""Display details for a specified volume."""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers
from SoftLayer import utils


def get_capacity(file_volume):
    """Get the capacity Gb with validate the data"""
    capacity = '0'
    if file_volume['capacityGb'] != '':
        capacity = "%iGB" % file_volume['capacityGb']
    return capacity


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('volume_id')
@environment.pass_env
def cli(env, volume_id):
    """Display details for a specified volume."""
    file_manager = SoftLayer.FileStorageManager(env.client)
    file_volume_id = helpers.resolve_id(file_manager.resolve_ids, volume_id, 'File Storage')
    file_volume = file_manager.get_file_volume_details(file_volume_id)
    file_volume = utils.NestedDict(file_volume)

    table = formatting.KeyValueTable(['Name', 'Value'])
    table.align['Name'] = 'r'
    table.align['Value'] = 'l'

    storage_type = file_volume['storageType']['keyName'].split('_').pop(0)
    table.add_row(['ID', file_volume['id']])
    table.add_row(['Username', file_volume['username']])
    table.add_row(['Type', storage_type])
    table.add_row(['Capacity (GB)', get_capacity(file_volume)])

    used_space = formatting.convert_sizes(file_volume.get('bytes_used', 0), "GB", False)
    table.add_row(['Used Space', used_space])

    if file_volume.get('provisionedIops'):
        table.add_row(['IOPs', file_volume['provisionedIops']])

    if file_volume.get('storageTierLevel'):
        table.add_row(['Endurance Tier', file_volume['storageTierLevel']])

    table.add_row(['Data Center', file_volume['serviceResource']['datacenter']['name']])
    table.add_row(['Target IP', file_volume['serviceResourceBackendIpAddress']])

    if file_volume['fileNetworkMountAddress']:
        table.add_row(['Mount Address', file_volume['fileNetworkMountAddress']])

    if file_volume['snapshotCapacityGb']:
        table.add_row(['Snapshot Capacity (GB)', file_volume['snapshotCapacityGb']])
        if 'snapshotSizeBytes' in file_volume['parentVolume']:
            table.add_row(['Snapshot Used (Bytes)', file_volume['parentVolume']['snapshotSizeBytes']])

    table.add_row(["# of Active Transactions", file_volume['activeTransactionCount']])

    if file_volume['activeTransactions']:
        for trans in file_volume['activeTransactions']:
            if 'transactionStatus' in trans and 'friendlyName' in trans['transactionStatus']:
                table.add_row(['Ongoing Transaction', trans['transactionStatus']['friendlyName']])

    table.add_row(['Replicant Count', "%u" % file_volume.get('replicationPartnerCount', 0)])

    if file_volume['replicationPartnerCount'] > 0:
        # This if/else temporarily handles a bug in which the SL API
        # returns a string or object for 'replicationStatus'; it seems that
        # the type is string for File volumes and object for Block volumes
        if 'message' in file_volume['replicationStatus']:
            table.add_row(['Replication Status', file_volume['replicationStatus']['message']])
        else:
            table.add_row(['Replication Status', file_volume['replicationStatus']])

        replicant_table = formatting.Table(['Id', 'Username', 'Target', 'Location', 'Schedule'])
        replicant_table.align['Name'] = 'r'
        replicant_table.align['Value'] = 'l'
        for replicant in file_volume['replicationPartners']:
            replicant_table.add_row([
                replicant.get('id'),
                utils.lookup(replicant, 'username'),
                utils.lookup(replicant, 'serviceResourceBackendIpAddress'),
                utils.lookup(replicant, 'serviceResource', 'datacenter', 'name'),
                utils.lookup(replicant, 'replicationSchedule', 'type', 'keyname')
            ])
        table.add_row(['Replicant Volumes', replicant_table])

    if file_volume.get('originalVolumeSize'):
        original_volume_info = formatting.Table(['Property', 'Value'])
        original_volume_info.add_row(['Original Volume Size', file_volume['originalVolumeSize']])
        if file_volume.get('originalVolumeName'):
            original_volume_info.add_row(['Original Volume Name', file_volume['originalVolumeName']])
        if file_volume.get('originalSnapshotName'):
            original_volume_info.add_row(['Original Snapshot Name', file_volume['originalSnapshotName']])
        table.add_row(['Original Volume Properties', original_volume_info])

    notes = f"{file_volume.get('notes', '')}"
    table.add_row(['Notes', notes])

    env.fout(table)



TEST_ALLOWED_HOST = {
    'id': 12345,
    'name': 'Test Allowed Host',
    'accountId': 1234,
    'credentialId': None,
    'createDate': '2020-01-01 00:00:01',
    'iscsiAclCredentials': {
        'id': 129,
        'allowedHostId': 12345,
        'subnetId': 12345678
    },
    'subnetsInAcl': [{
        'id': 12345678,
        'accountId': 1234,
        'networkIdentifier': '10.11.12.13',
        'cidr': '14',
        'billingRecordId': None,
        'parentId': None,
        'networkVlanId': None,
        'createDate': '2020-01-02 00:00:01',
        'modifyDate': None,
        'subnetType': 'SECONDARY_ON_VLAN',
        'restrictAllocationFlag': 0,
        'leafFlag': 1,
        'ownerId': 1,
        'ipAddressBegin': 129123,
        'ipAddressEnd': 129145,
        'purgeFlag': 0
    }]
}

getObject = TEST_ALLOWED_HOST

getSubnetsInAcl = [{
    'id': 12345678,
    'accountId': 1234,
    'networkIdentifier': '10.11.12.13',
    'cidr': '14',
    'billingRecordId': None,
    'parentId': None,
    'networkVlanId': None,
    'createDate': '2020-01-02 00:00:01',
    'modifyDate': None,
    'subnetType': 'SECONDARY_ON_VLAN',
    'restrictAllocationFlag': 0,
    'leafFlag': 1,
    'ownerId': 1,
    'ipAddressBegin': 129123,
    'ipAddressEnd': 129145,
    'purgeFlag': 0
}]

assignSubnetsToAcl = [
    12345678
]

removeSubnetsFromAcl = [
    12345678
]

setCredentialPassword = True



"""Edit ip note"""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--note', help="set ip address note of subnet")
@environment.pass_env
def cli(env, identifier, note):
    """Set the note of the ipAddress"""

    data = {
        'note': note
    }
    mgr = SoftLayer.NetworkManager(env.client)
    ip_id = None
    if str.isdigit(identifier):
        ip_id = identifier
    else:
        ip_object = mgr.get_ip_by_address(identifier)
        ip_id = ip_object.get('id')
    mgr.set_subnet_ipddress_note(ip_id, data)
